/* math_parser.c */
/* Math formula parser 0.5 beta - main functions
 * Written in C89 (with optional C99 parts)
 * Copyright by Andrea Oster, 2007/2008
 */
 /**************************************************************************
 *   Copyright (C) 2007,2008 by Andrea Oster				   *
 *   ostera@googlemail.com						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2, as   *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <float.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <assert.h>

#include "math_parser.h"
#include "math_parser_helperfns.h"
#include "math_parser_compat.h"

#ifndef USE_C99
#define inline /*nothing*/
#ifdef USE_LDBL
#define double long double
#define DBL_EPSILON LDBL_EPSILON
#endif /*USE_LDBL*/
#endif /*USE_C99*/ 

const struct func fn={
			{
				"",
				"sin","cos","tan",
				"asin","arcsin","acos","arccos","atan","arctan",
				"cot","acot","arccot",
				"sinh","cosh","tanh",
				#ifdef USE_C99
				"asinh","acosh","atanh",
				#else
				"","","",
				#endif
				"lg","ln","exp","sqrt","fac","abs","ceil","floor","round",
				"min","max"
			}
		};

/* declarations of some internal functions */
inline static void sumdiffparse(const char*,subfunction*);
inline static void faktparse(const char*, subfunction*);
inline static void quotparse(const char*, subfunction*);
static void sumfaktparse(const char*, subfunction*,char);
static void powparse(const char*, subfunction*);
static void was_eintragen(const char *what, subfunction *fn, uint32_t num);
static bool powlast(const char*);	
inline static double factorial(uint16_t in);
static int preprocess(char* *str);
static void moduloparse (const char* input, subfunction *output);
static int16_t hasfn(const char* what, char* *in);
static void funcparse(const char*, subfunction*);

static bool simpleparse(subfunction *in){
/* Parses one subfunction into its summands/factores/... (depending on strdiv), placing this in next 
 * fncache is a function parsing one string to one subfunction
 */
	uint32_t i;
	void (*fncache)(const char*,subfunction*);
	if (in->num==0) return false;
	in->next=NULL;
	for (i=0;i<in->num;++i){
		if(!in->simplissimus[i]){
			vor:
			if(strdiv(in->nums_t[i].str,NULL,"+-","()")>1)
				fncache=sumdiffparse;
			else if (strdiv(in->nums_t[i].str,NULL,"*","()")>1)
				fncache=faktparse;
			else if (strdiv(in->nums_t[i].str,NULL,"/","()")>1)
				fncache=quotparse;
			else if (strdiv(in->nums_t[i].str,NULL,"%","()")>1)
				fncache=moduloparse;
			else if (strdiv(in->nums_t[i].str,NULL,"^","()")>1){
				if(in->nums_t[i].str[0]=='-'){
					in->nums_t[i].str=(char*)realloc(in->nums_t[i].str,strlen(in->nums_t[i].str)+3);
					strins(in->nums_t[i].str,1,"1*");
					fncache=faktparse;
				}
				else fncache=powparse;
			}
			else if (hasfn(in->nums_t[i].str,NULL)!=0)
				fncache=funcparse;
			else if	(in->nums_t[i].str[0]=='-')
				fncache=sumdiffparse;
			else{
					preparse(in->nums_t[i].str);
					(void)removespaces(in->nums_t[i].str);
					in->simplissimus[i]=!canbesimplified(in->nums_t[i].str);
					if(in->simplissimus[i])
						continue;
					goto vor;
			}
			if (!(in->next = (subfunction*) realloc(in->next,(i+1)*sizeof(subfunction)))) return false;/*set errno?*/
			in->next[i].prev=in;
			(*fncache)(in->nums_t[i].str,&in->next[i]);
		}
	}
	if(!in->next) return false;/* stop recursing if all is simplissimus */
	return true;
}


static void parselevel(subfunction *in){
/* parses tree level with simpleparse */
	subfunction* level;
	uint32_t i,j;
	bool recurse;
	for (i=0;i<in->prev->num;++i){
		recurse=false;
		if (!in->prev->simplissimus[i]){
			level=&in->prev->next[i];
			if(simpleparse(level))recurse=true;
			for(j=0;j<level->num;++j)
				if (recurse && !level->simplissimus[j]) parselevel(&level->next[j]);
		}
	}
}

static void parse(char* input, subfunction *output){
	memset(output,0,sizeof(subfunction));
	/* Term einlesen */
	output->num=1;
	output->nums_t = (subfn_nums*) malloc(sizeof(subfn_nums));
	output->nums_is_dbl=(bool*)calloc(1,sizeof(bool));
	output->simplissimus = (bool*) malloc(sizeof(bool));
	was_eintragen(input,output,0);
	output->type='s';
	{ 
/*reduced parselevel(); needed because ->prev is not defined for the root subfunction*/
		if (simpleparse(output))
			parselevel(output->next);
	}
}


void param_set_var(struct params *par,char name,double value){/*TODO rewrite*/
	uint16_t i;
	/*if (par->num==0 || par->values==NULL) param_init(par);*/
	for (i=0;i<par->num;++i) /* nach Variable mit Namen name suchen, sonst neue hinzufuegen */ 
		if (i<par->num && par->names[i]==name){
			par->values[i]=value;
			return;
		}
	++(par->num);
	par->names = (char*)realloc(par->names,(par->num)*sizeof(char));
	par->values = (double*)realloc(par->values,(par->num)*sizeof(double));
	par->names[par->num-1]=name;
	par->values[par->num-1]=value;
}

void param_set_var_by_num(struct params *par, int16_t num, double value){
/* Compat function for use in bindings to programming langs that don't allow pointer being an accessed as an array, like Delphi */
	if (num<0 || num>par->num) return;
	par->values[num]=value;
}

double param_get_var(const struct params *par, char name){
	uint16_t i;
	for (i=0;i<par->num;++i)
		if (par->names[i]==name) break;
	return par->values[i];
}

bool param_isset(const struct params *par,char name){
	uint16_t i;
	for (i=0;i<par->num;++i)
		if (par->names[i]==name) return true;
	return false;
}

double exec_simplestring(const char* str, const struct params *par){
	double result;
	char *endval;
	
	if (strlen(str)==1 && isalpha(*str))
		return (param_isset(par,*str) ? param_get_var(par,*str) : 0);
	else if(strlen(str)==2 && isalpha(str[1])){
		if(*str=='-')
			return  -(param_isset(par,str[1]) ? param_get_var(par,str[1]) : 0);
		else if (*str==' ' || *str=='+'){
			return (param_isset(par,str[1]) ? param_get_var(par,str[1]) : 0);
		}
		else return 0;
	}
	else{
		const char *tmp=&str[0];
		return strtod(tmp,&endval);
	}
}

static double func_exec(double in,int16_t ident,bool *ok,bool *spruenge){
	double sign=(ident>0 ? 1 : -1);
	/* spruenge has to be initialized by the caller */
	switch(abs(ident)){ /*note: in is in radient (for sinus and friends) */
		/*TODO complete error handling */
		case 0:	return sign*in;
		case 1:	return sign*sin(in);
		case 2:	return sign*cos(in);
		case 3:
			if (fabs(fmod(in,M_PI)-M_PI_2)>DBL_EPSILON)
				return sign*tan(in);
			*ok=false;
			return 0;
		case 4: case 5:
			if(fabs(in)<=1)
				return sign*asin(in);
		case 6: case 7:
			if(fabs(in)<=1)
				return sign*acos(in);
			*ok=false;
			return 0;
		case 8: case 9:return sign*atan(in);
		case 10:
			if(sin(in)!=0)
				return sign*cos(in)/sin(in);
			*ok=false;
			return 0;
		case 11: case 12:
			if(atan(in)!=0)
				return sign*1.0/atan(in); /* _this_ way? */
			*ok=false;
			return 0;
		case 13:return sign*sinh(in);
		case 14:return sign*cosh(in);
		case 15:return sign*tanh(in);
		#ifdef USE_C99
		case 16:return sign*asinh(in);
		case 17:return sign*acosh(in);
		case 18:return sign*atanh(in);
		#endif
		case 19:
			if (in>0)
				return sign*log10(in);
		case 20:
			if (in>0)
				return sign*log(in);
			*ok=false;
			return 0;
		case 21:return sign*exp(in);
		case 22:
			if (in>=0)
				return sign*sqrt(in);
			*ok=false;
			return 0;
		case 23:
			*spruenge=true;
			if (in>=0 && fmod(in,1)==0)
				return sign*factorial(in);
			*ok=false;
			return 0;
		case 24:
			return sign*fabs(in);
		case 25:
			*spruenge=true;
			return sign*ceil(in);
		case 26:
			*spruenge=true;
			return sign*floor(in);
		case 27:
			*spruenge=true;
			return sign*round(in);
		default:return sign*in;
		
	}
}

static double calc(const subfunction *func,const struct params *p,bool *ok,bool *spruenge){
	double result;
	uint32_t i;
	*ok=true;/*spruenge has to be set false by the caller before calling calc()*/
	result=(func->type=='s' ? 0.0 : 1.0);
	if(func->type==28 || func->type==29){
		double tmp;
		result= func->type==28 ? HUGE_VAL : -HUGE_VAL;
		for(i=0;i<func->num;++i){
			tmp=( func->simplissimus[i] ? ( func->nums_is_dbl[i] ? func->nums_t[i].d : exec_simplestring(func->nums_t[i].str,p) ) : calc(&func->next[i],p,ok,spruenge) );
			result=(func->type==28 ? MIN(result,tmp) : MAX(result,tmp));
		}
	}
	else for(i=0;i<func->num;++i){
		double looppass,tmpval;
		if(func->type=='s') looppass=0;
		else /*if(func->type=='f')*/looppass=1;
		if(func->type=='m') *spruenge=true;

		if (func->simplissimus[i] || !func->next){
			tmpval=func->nums_is_dbl[i] ? func->nums_t[i].d  : exec_simplestring(func->nums_t[i].str,p);
			if (func->type=='s')
				looppass+=tmpval;
			else if (func->type=='f'||(func->type=='q' /*&& i==0*/))
				looppass*=tmpval;
			else if (func->type=='p' || func->type=='m')
				looppass=tmpval;
			else	looppass=func_exec(tmpval,func->type,ok,spruenge);
		}
		else{
			tmpval=calc(&func->next[i],p,ok,spruenge);
			if (func->type=='s')
				looppass+=tmpval;
			else if(func->type=='f'|| (func->type=='q'/* && i==0*/))
				looppass*=tmpval;
			else if(func->type=='p' || func->type=='m')
				looppass=tmpval;
			else	looppass=func_exec(tmpval,func->type,ok,spruenge);
			if (!*ok) return 0;
		}
		if(func->type=='s')
			result+=looppass;
		else if(func->type=='f'|| (func->type=='q' && i==0))
			result*=looppass;
		else if(func->type=='q') {
				if (looppass!=0)
					result/=looppass;
				else *ok=false;
			}
		else if (func->type=='p')
			result = (i==0 ? looppass : pow(result,looppass));
		else /*m*/
			result = (i==0 ? looppass : fmod(result,looppass));

		if (func->type=='f' && result==0) break;
			/* 0*x stays 0, so there is no need to calculate the product further*/
#ifdef USE_C99
		if(!isfinite(result) || !isfinite(looppass)){
			*ok=false;
			return 0;
		}
#endif
	}
	return result;
}

struct result subfn_evaluate(const subfunction *function, const struct params *par){
	struct result res;
	res.spruenge=false;
	res.value=calc(function,par,&res.defined,&res.spruenge);
	return res;
}

int32_t subfn_readin(subfunction *function,struct params *par, const char* input){
	int32_t i=0;
	char **term;

	if (strdiv(input,&term,"=","")!=2) return -1;

	while (term[0][i++]!='('&& term[0][i]); 
	while (term[0][i]!=')'&& term[0][i]){
		if(isalpha(term[0][i]))param_set_var(par,term[0][i],0);
		++i;
	}		

	term[1][0]=' '; /* away w/ leading '=' */
	(void)preprocess(&term[1]);
	(void)removespaces(term[1]);
	i=validate(term[1],par);
	if(i!=0)	return (int32_t)strlen(term[0])+i+1; /* return fehlerort */
	parse(term[1],function);
	free(term[0]);
	free(term[1]);
	free(term);
	return 0;
}

/* zwei Wrapperfunktionen */
struct result auswerten(const subfunction *function, const struct params *par){
	return subfn_evaluate(function,par);
}
int32_t einlesen(subfunction *function,struct params *par, const char* input){
	return subfn_readin(function,par,input);
}

inline static void moduloparse(const char* input, subfunction *output){
	sumfaktparse(input,output,'m');
}
inline static void sumdiffparse(const char* input, subfunction *output){
	sumfaktparse(input,output,'s');
}
inline static void faktparse(const char* input, subfunction *output){
	sumfaktparse(input,output,'f');
}
inline static void quotparse(const char* input,subfunction *output){
	sumfaktparse(input,output,'q');
}

static void sumfaktparse(const char* input, subfunction *output,char what){
	char **sumfaktoren;
	uint32_t i,zaehler;
	char sign[3];
	
	if(what=='s')strcpy(sign,"+-");
	else if(what=='f')strcpy(sign,"*");
	else if(what=='q')strcpy(sign,"/");
	else if(what=='m')strcpy(sign,"%");

	zaehler=strdiv(input, &sumfaktoren,sign,"()");
	output->type = what;
	output->num = zaehler;
	output->nums_t=(subfn_nums*)malloc(zaehler*sizeof(subfn_nums)); 
	output->nums_is_dbl=(bool*)malloc(zaehler*sizeof(bool));
	output->simplissimus = (bool*) malloc(zaehler*sizeof(bool));
	if(!output->nums_t || !output->simplissimus){
		free(output->nums_t);
		free(output->simplissimus);
		return;
	}
	for (i=0;i<zaehler;++i){
		/* now cut away the * or / or % : */
		if (sumfaktoren[i][0]=='/' || sumfaktoren[i][0]=='*' || sumfaktoren[i][0]=='%')sumfaktoren[i][0]=' ';
		was_eintragen(sumfaktoren[i],output,i);
		free(sumfaktoren[i]);
	}
	free(sumfaktoren);
}

static int powlast_part(const char *what,const char *tmp,bool forward){
	int klammerzaehler=0;
	while(true){
		tmp+= forward ? 1 : -1;
		if(!*tmp || tmp==what) break; 
		switch (*tmp){
			case '(': ++klammerzaehler;break;
			case ')': --klammerzaehler;break;
			case '+': case '-':
			case '*': case '/':
			case '%':
				  if(klammerzaehler==0)return -1;
		}
	}
	return klammerzaehler;
}
static bool powlast(const char *what){
/* Ergebniswert: soll "pow_fork" gemacht werden */
	char *help,*tmp;
	int klammerzaehler=0;
	help=strchr(what,'^');
	if (help==NULL) return false;
	/* first look after  '^' */
	if (help[1]!='('){ 
		if (strpbrk(help,"+-*/")!=NULL) return false;
	}else{
		/* keine + - * /   % ausserhalb der Klammern (klammerzaehler==0)*/
		if(powlast_part(what,help,true)==-1)return false;
	}
	/* then before it */
	if(powlast_part(what,help,false)==0)return true;
	return false;
}

static void powparse(const char input[], subfunction *output){
	uint32_t i=0,slen=0,powzaehler;
	char **pows=NULL,**out;
	/* if the string is a power, it's split ; then we look if the potence itself can be divided in potences */
	powzaehler=strdiv(input,&pows,"^","()");
	if(!pows) return;
	if(!(out=(char**)malloc(sizeof(char*)))){return;}/* characters before first ^ */
	out[0]=strdup(pows[0]);
	if (powzaehler!=1)
	for (i=1;i<powzaehler;++i){
		out=(char**)realloc(out,(i+1)*sizeof(char*));
		slen+=strlen(out[i-1]);
		if (powlast(&input[slen+1]))
			out[i]=strdup(pows[i]);
		else{/* no  further split : copy rest of string in last string of pows */ 
			out[i]=strdup(&input[slen+1]);
			break;
		}
	}
	slen=i+1;/*other use of slen */
	
	for (i=0;i<powzaehler;++i)
		free(pows[i]);
	free(pows);
	
	output->type='p';
	output->num=slen;
	output->nums_t=(subfn_nums*)malloc(slen*sizeof(subfn_nums));
	output->nums_is_dbl=(bool*)malloc(slen*sizeof(bool));
	output->simplissimus = (bool*) malloc(slen*sizeof(bool));
	for (i=0;i<slen;++i){
		if (out[i][0]=='^') out[i][0]=' ';
		if (out[i][0]=='-'){
		}
		/* now cut away the - or + : , if / it comes in a quot list */
	/*	if (out[i][0]=='+'  || out[i][0]=='-')out[i][0]=' ';*/
		was_eintragen(out[i],output,i);
		free(out[i]);
	}
	free(out);
}

static int16_t hasfn(const char what[], char* in[]){
	int16_t i,klammerzaehler;
	const char* tmp,* tmp2,*search;
	bool minus=false;

	klammerzaehler=1;
	if(*what=='-'){
		minus=true;
		++what;
	}
	tmp=what;
	tmp2=what;
	for (i=1;i<FN_IDENTNUM;++i){
		if(strlen(fn.name[i])>0 && (search=strstr(what,fn.name[i]))==what && search!=NULL && search[strlen(fn.name[i])]=='('){
			tmp+=strlen(fn.name[i]);
			tmp2+=strlen(fn.name[i]);
			while(*++tmp2)
				switch (*tmp2){
					case ')':
						--klammerzaehler; break;
					case '(':
						++klammerzaehler; break;
					default:
						if (klammerzaehler==0 && tmp2[1]!='\0') return 0;
				}
		 	if (in!=NULL){
				*in=(char*) calloc((strlen(tmp)),sizeof(char));
				strpartcpy(tmp,1,strlen(tmp)-2/* ohne ')' am ende*/,*in);
			}
			return (minus ? -i : i);
		}
	}
	return 0;
}
static void funcparse(const char *input,subfunction *output){
	int16_t i,argc=1,count=1;
	char *tmp,**out=NULL;
	int fnum=hasfn(input,&tmp);
	if(!fnum)return;
	if(abs(fnum)>=28) argc=2; 
	
	output->type=fnum;
	output->num=(argc>1 ? (count=strdiv(tmp,&out,";","")): 1);
	output->simplissimus=(bool*)malloc(count*sizeof(bool));
	output->nums_is_dbl=(bool*)malloc(count*sizeof(bool));
	output->nums_t=(subfn_nums*)malloc(count*sizeof(subfn_nums));
	if(argc>1){
		for(i=0;i<count;++i){
			if(out[i][0]==';')out[i][0]=' ';
			was_eintragen(out[i],output,i);
			free(out[i]);
		}
		free(out);
	}
	else was_eintragen(tmp,output,0);
}

static void was_eintragen(const char *what, subfunction *fn, uint32_t num){
	double d;
	char *endptr=NULL;
	char *wcopy=strdup(what);

	preparse(wcopy);
	removespaces(wcopy);
	preparse(wcopy);
	removespaces(wcopy);
	
	if((fn->simplissimus[num]=!canbesimplified(wcopy))){
	 	d=strtod(wcopy,&endptr);
		if(!endptr || strlen(endptr)==0){
			fn->nums_t[num].d=d;
			fn->nums_is_dbl[num]=true;
		}
		else{
			fn->nums_t[num].str=strdup(wcopy);
			fn->nums_is_dbl[num]=false;
		}
	}
	else	fn->nums_t[num].str=strdup(wcopy);
	
	fn->next=NULL;
	free(wcopy);
}


#define STR(s) (#s)
static int preprocess(char* *str){
/* Only for the expression side */
	const char* defines[]={
		"PI"
	};
	const char* values[]={
		STR(M_PI)
	};
	int i;
	char *tmp, *copy,*strend;
	copy=strdup(*str);
	for(i=0;i<1;++i){
		tmp=strstr(copy,defines[i]);
		if(tmp==NULL)continue;
		tmp+=strlen(defines[i]);
		*(tmp-strlen(defines[i]))='\0';
		strend=strdup(tmp);
	
		copy=realloc(copy,strlen(*str)+1-strlen(defines[i])+strlen(values[i]));
		strcat(copy,values[i]);
		strcat(copy,strend);
		free(strend);
	}
	if (!(*str=(char*)realloc(*str,strlen(copy)+1))){
		return -1;/*errno?*/
	}
	strcpy(*str,copy);
	free(copy);
	return 0;
}

void subfn_free(subfunction *sfn){
	uint32_t i;
	bool a=false;
	if (!sfn || sfn->num==0) return;
	for (i=0;i<sfn->num;++i){
		free(sfn->nums_is_dbl[i] ? NULL : sfn->nums_t[i].str);
		if (sfn->next!=NULL && !sfn->simplissimus[i]){subfn_free(&sfn->next[i]);a=true;}
	}
	free(sfn->nums_is_dbl);
	free(sfn->nums_t);
	if(a)free(sfn->next);
	free(sfn->simplissimus);
	sfn->simplissimus=NULL;
	sfn->num=0;
}


char* subfn_get_fnstr(subfunction *sfn){/*TODO testing*/
	uint32_t i,size=1;
	int16_t x;
	char *res=(char*)malloc(10);
	if (!res) return NULL;
	*res='\0';
	if(sfn==NULL || sfn->nums_t==NULL) {free(res);return NULL;}
	if(sfn->type<30){strcat(res,fn.name[sfn->type]);strcat(res,"(");}
	for(i=0;i<sfn->num;++i){
		size+=strlen(sfn->nums_t[i].str);/*geht dat so?*/
		res=(char*)realloc(res,size);
		strcat(res,sfn->nums_t[i].str);
	}
	if(sfn->type<30){strcat(res,")");}
	return res;
}

static subfunction sfn_copy_(const subfunction *from){
	subfunction to;
	uint32_t i;
	to.simplissimus=(bool*)malloc(from->num*sizeof(bool));
	to.nums_t=(subfn_nums*)malloc(from->num*sizeof(double));
	to.nums_is_dbl=(bool*)malloc(from->num*sizeof(bool));
	memcpy(to.nums_is_dbl,from->nums_is_dbl,from->num);
	if(!to.simplissimus || !to.nums_t){
		free(to.simplissimus);free(to.nums_t);
		to.num=0;return to;
	}
	to.num=from->num;
	to.type=from->type;
	to.next=(subfunction*)malloc(from->num*sizeof(subfunction));
	for (i=0;i<from->num;++i){
		
		if((to.nums_is_dbl[i]/*=from->nums_is_dbl[i]*/))
			to.nums_t[i].d=from->nums_t[i].d;
		else	to.nums_t[i].str=strdup(from->nums_t[i].str);
			
		to.simplissimus[i]=from->simplissimus[i];
		if(from->next && !from->simplissimus[i])
			to.next[i]=sfn_copy_(&from->next[i]);
	}
	return to;
}

void sfn_copy_part(subfunction* to, subfunction *from,uint32_t t_num, uint32_t f_num){
	if(!to->nums_is_dbl)to->nums_is_dbl=(bool*)malloc(to->num*sizeof(bool));
	to->nums_is_dbl[t_num]=from->nums_is_dbl[f_num];
	if(!to->nums_t)	to->nums_t=(subfn_nums*)malloc(to->num*sizeof(double));
	if(to->nums_is_dbl[t_num])	to->nums_t[t_num].d=from->nums_t[f_num].d;
	else				to->nums_t[t_num].str=strdup(from->nums_t[f_num].str);
	if(!to->simplissimus)
			to->simplissimus=(bool*)malloc(to->num*sizeof(bool));
	to->simplissimus[t_num]=from->simplissimus[f_num];
	if(to->next==NULL && from->next!=NULL)	to->next=(subfunction*)malloc(to->num*sizeof(subfunction));
	if(from->next && !from->simplissimus[f_num])	to->next[t_num]=sfn_copy_(&from->next[f_num]);
}

static void subfn_jump_fn_pars_part(subfunction* level,subfunction* *out,bool* in_out,int *num){
	uint32_t i=0;
	if(level->type=='m'){
		(*num)++;
		*out=(subfunction*)recalloc(*out,(*num)-1,*num,sizeof(subfunction));
		(*out)[(*num)-1]=sfn_copy_(level);
		*in_out=true;
	}
	else if (abs(level->type)>=25 && abs(level->type<=27)){
				(*num)++;
				*out=(subfunction*)recalloc(*out,(*num)-1,*num,sizeof(subfunction));
				/*sfn_copy_part(&(*out)[(*num)-1],level,(*out)[(*num)-1].num-1,i);*/
				(*out)[*(num)-1]=sfn_copy_(level);
				*in_out=true;
	}
	else{
				*in_out=false;
	}
}

void subfn_give_subfn_jump_fn_pars(subfunction* *out/* has to be calloc(1) by caller */,const subfunction* sfn,int *num){
	uint32_t i;/* num is to be inited 0 by the caller - because of recursion */
	subfunction *level=sfn;
	bool in_out=false;
/*	subfunction * */ *out=(subfunction*)recalloc(*out,1,MAX(*num,1),sizeof(subfunction));

	subfn_jump_fn_pars_part(level,out,&in_out,num);
	for(i=0;i<level->num;++i)
		if (!level->simplissimus[i] && !in_out && level->next){
			level=&level->next[i];
			subfn_give_subfn_jump_fn_pars(out,level,num);
		}

	if(*num==0){
		subfn_free(*out);
		free(*out);
		*out=NULL;
	}
}

/* calc functions */
/* make this inline (C99) or a macro */
inline static double factorial(uint16_t in){
	uint16_t i;
	double res=1;
	/*preconditions?*/
	for (i=2;i<=in;++i)
		res*=i;
	return res;
}



double integrate_qad/*quick and dirty*/(subfunction* s, struct params *p,double range1,double range2){
	double r=0.0,i;
	struct result res;
	for(i=range1;i<range2;i+=0.000100){
		if(p->num>0)p->values[0]=i;
		res=subfn_evaluate(s,p);
		r+=res.value/10000;
	}
	return r;
}
