/* math_parser.h */
/* Description of the expected formula fornat
 * ------------------------------------------
 * funcname(paramlist)=expression
 * - funcname is quite free, may be as long as you wish (as long you don't exceed platform pointer limits) but may not contain a '('
 * - paramlist: param1,param2 ; int16_t number of params allowed, separated by ','
 * - parameters: consist of 1 (VARLEN) character, which must either match isupper() or islower()
 * - expression: like common mathematics, but (common for computer implementations) one may not omit the '*' in e.g. "4*x" and you may write ".3" for "0.3"
 *   	allowed operands: + - / * %
 * - func (sin etc.) 1 parameter except min,max(theoretically infinite, separated by ';') 
 * - note: currently only () brackets supported
 */
#ifndef MATH_PARSER_H

#define MAXVAR 52
#define VARLEN 1
#define FN_IDENTNUM 30

#ifndef M_PI /* from glibc 2.5's math.h (LGPL 2.1+) */
#define M_PI	3.14159265358979323846	/* pi */
#define M_PI_2	1.57079632679489661923	/* pi/2 */
#endif

#ifdef USE_C99
#include <stdint.h>
#include <stdbool.h>
#else
typedef short int16_t;
typedef unsigned short uint16_t;
typedef int int32_t;
typedef unsigned int uint32_t;

typedef char bool;
enum {false=0,true=1};
#endif

#undef MAX
#undef MIN
#define MAX(a,b) ((a)>(b) ? (a) : (b))
#define MIN(a,b) ((a)<(b) ? (a) : (b))

struct result{
	double value;
	bool defined;
	bool spruenge;
};

struct params{
	char* names;
	double* values;
	uint16_t num;
};

typedef union _subfn_nums{
	char *str;
	double d;
}subfn_nums;

typedef struct subfunction{
	signed char type; /*(s)um/diff, (f)actor, (q)uotient, (p)ower or (m)odulo */
	/* others: normal = 0,functions (like sin())>0  */	
	
	uint32_t num;
	union _subfn_nums *nums_t;
	bool *nums_is_dbl;

	bool *simplissimus; /* if nums_t[x] can not be more simplified */
	struct subfunction* prev;/* Note: prev is a pointer to _one_ struct; next is an array of such */ 
	struct subfunction* next;
	
}subfunction;

struct func{ /* functions like sin() ,not f(x)=... */
	char* name[FN_IDENTNUM];
	/*int identifier[FN_IDENTNUM]; /* used in subfunction.nexttype */
};

struct result subfn_evaluate(const struct subfunction *function, const struct params *par);
int32_t subfn_readin(struct subfunction *function, struct params *par, const char* input);
/*wrapper functions:*/
struct result auswerten(const struct subfunction *function, const struct params *par);
int32_t einlesen(struct subfunction *function, struct params *par, const char* input);

double exec_simplestring(const char *str, const struct params *par);

void subfn_free(subfunction *sfn);
/* note: subfn_free frees only the elements of the pointed subfunction, not itself, which means you have to call free(sf) yourself */

void subfn_give_subfn_jump_fn_pars(subfunction* *,const subfunction*,int*);

void param_set_var(struct params*,char,double);
double param_get_var(const struct params*,char);
bool param_isset(const struct params*,char);

double integrate_qad(subfunction*,struct params*,double,double);

#define MATH_PARSER_H MATH_PARSER_H
#endif /* MATH_PARSER_H */
