/* math_parser_gtk2.c */
/* GTK2 GUI for the math_parser 0.5
 * Copyright (C) Andrea Oster, 2007/2008
 * Languages: German, English (see .._lang.c)
 * Programming Language: C(89 plus a few 99)
 * License: GPL v2 only
 * Requires at least GTK 2.4
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include <locale.h>

#ifdef __WIN32__
#include <windows.h>
#endif

#include "math_parser_compat.h"
#include "math_parser_gtk2.h"
#include "math_parser_gtk2_mainwindow.h"
#include "math_parser_gtk2_lang.h"
#include "math_parser_gtk2_draw.h"
#include "math_parser_gtk2_misc.h"
#include "math_parser_gtk2_helperfns.h"

static GtkWidget /* *label1[MAX_PARAMS], *entry1[MAX_PARAMS], *fentry,*rlabel,*/ *graph,*gwindow,*valuetable;
static graph_cache gcache;
/*static GtkAccelGroup *menu_accel[3];*/
static GtkWidget *box[MAX_PARAMS-1],*p2entry[MAX_PARAMS-1][3],*label2[MAX_PARAMS-1],*colorbutton,*errlabel;
int skalx,skaly;/*pixels per unit*/
/*static uint16_t curr_eq; /*which equation is current[0..MAX_EQUATIONS-1] * /
uint16_t curr_eq_get(void){return curr_eq;}*/
static struct xy_float zeropoint_rel;/*relative coords on GtkDrawingArea: (virt 0pt)/allocation->w/h */
int zero_x(void){return zeropoint_rel.x*graph->allocation.width;}
int zero_y(void){return zeropoint_rel.y*graph->allocation.height;}
static GtkTreeIter iter;
static GtkListStore *listst;
static graph_context gc;
static bool drawsingle,blackwhite;
double singlevals[MAX_EQUATIONS][MAX_PARAMS];
/*static double integr_vals[MAX_EQUATIONS][2];*/
parser_context *pc;

extern int langnum;
extern char *langs[5], *langs_short[5];
extern struct{
	struct labels *l[3];
}labelhandle;
extern struct labels DE,EN,FR;
struct labels *labels_lang;

gboolean zeichnen(void);
#define REDRAW_GRAPH zeichnen()
void redraw_graph(void){REDRAW_GRAPH;}

struct{
	bool options,points,verticals,areas;
}visible;

void reset_all(void);

gboolean valuetable_set(void);
static gboolean valuetable_export(char* filename);
/* window function decls:
 *  they are concipied w/ static vars:
 *  - first call creates actual window
 *  - further calls re-show the window (if necessary) and update language-dependent stuff */
gboolean valuetable_init(void);
static int show_gwindow(void);
void show_gwindow_full(void);
void main_window(void);/* in _mainwindow.c*/
gboolean shell_window(void);

static int saveini(const char*,const parser_context*);
static int openini(const char*);

static int openmgd(const char*);

gboolean print_wrapper(void);
int changelang(GtkWidget*);
gboolean optiondialog(void);
gboolean aboutbox(void);


gboolean graph_export_wrapper(void){
	char* pat[1][2]={"*.png","Portable Network Graphics (*.png)"};
	GdkPixbuf *pb=gdk_pixbuf_get_from_drawable(NULL,gcache.pm,
			gdk_colormap_get_system(),0,0,0,0,gcache.width,gcache.height);
	char *filename=filedialog_generic(false,"Graph export",pat,1);
	if(filename){
		if(strcmp(filename+strlen(filename)-4,".png")!=0){
			filename=g_realloc(filename,strlen(filename)+5);
			strcat(filename,".png");
		}
	if(file_exists_dialog(filename))
			gdk_pixbuf_save(pb,filename,"png",NULL,"compression","9",NULL);	
	}
	free(filename);
}

gboolean vtab_export_wrapper(void){
	char *pat[1][2]={"*.csv","Comma-separated values (*.csv)"};
	
	char *filename=filedialog_generic(false,"Tab export",pat,1);
	if(filename && strcmp(filename+strlen(filename)-4,".csv")!=0){
		filename=g_realloc(filename,strlen(filename)+5);	
		strcat(filename,".csv");
	}
	if(file_exists_dialog(filename))
		valuetable_export(filename);
	free(filename);
	return TRUE;
}
gboolean save_wrapper(void){
	char *pat[1][2]={"*.ini","Andrea's Funktionsrechner (*.ini)"};
	char *filename=NULL;
	
	filename=filedialog_generic(false,"Save",pat,1);
	if(filename && strcmp(filename+strlen(filename)-4,".ini")!=0){
		filename=g_realloc(filename,strlen(filename)+5);	
		strcat(filename,".ini");
	}
	if(file_exists_dialog(filename))
		saveini(filename,pc);
	return TRUE;
}
gboolean open_wrapper(void){
	char *pat[][2]={{"*.ini","Andrea's Funktionsrechner (*.ini)"},
#ifdef USE_MGD2INI
			{"*.mgd","MatheGrafix 6/7 (*.mgd)"},
#endif
			{"*","All Files (*)"}};
	char *filename=filedialog_generic(true,"Open",pat,2);

	if (filename){
#ifdef USE_MGD2INI
		if(strcmp(filename+strlen(filename)-4,".mgd")==0)openmgd(filename);
		else
#endif
		switch(openini(filename)){
			case 0:	break;
			case -1:	ShowErrorDlg("Wrong file format"); break;
			case -2:	ShowErrorDlg("File not readable"); break;
			}
	}
	free(filename);
	return TRUE;
}
gboolean new_wrapper(void){
	GtkWidget *dialog=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,labels_lang->confirm_new);
	if(gtk_dialog_run((GtkDialog*)dialog)==GTK_RESPONSE_ACCEPT)
		reset_all();
	gtk_widget_destroy(dialog);
	return FALSE;
}
gboolean exit_wrapper(void){
	GtkWidget *dialog=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,labels_lang->confirm_exit);
	if(gtk_dialog_run((GtkDialog*)dialog)==GTK_RESPONSE_YES)
		exit(EXIT_SUCCESS);
	gtk_widget_destroy(dialog);
	return TRUE;
}
gboolean help_wrapper(void){
	ShowErrorDlg("Sorry, but help is not available in this beta version");
	return TRUE;
}

static gboolean showgraphpos(GtkWidget *w,GdkEventMotion *pos,gpointer in);
static gboolean change_eq_nr();
static gboolean dborders_eq(GtkWidget *w,GtkEntry *e[2]);
static gboolean graph_move(GtkWidget *widget,GdkEventButton *event);

static void pixmap_to_graph(void){
	GdkGC *gc=gdk_gc_new(graph->window);
	if (gcache.pm)gdk_draw_drawable(graph->window,gc,gcache.pm, 0,0, 0,0, -1,-1);
	g_object_unref(gc);
}
gboolean zeichnen(void){
	int i=0,j;
	GdkColor white={0,65535,65535,65535};
	
	if(gcache.pm) 	g_object_unref(gcache.pm);
	if(gcache.pango)g_object_unref(gcache.pango);
	gdk_drawable_get_size(graph->window,&gcache.width,&gcache.height);
	gcache.pm=gdk_pixmap_new(graph->window,gcache.width,gcache.height,-1);
	GdkGC *gc1=gdk_gc_new(graph->window);
	gdk_gc_set_rgb_fg_color(gc1,&white);
	gdk_draw_rectangle(gcache.pm,gc1,true,0,0,gcache.width,gcache.height);
	g_object_unref(gc1);
	gcache.pango=gtk_widget_create_pango_layout(graph,"");

	draw_gdk_areas(&gcache,skalx,skaly,blackwhite,pc);
	draw_gdk_circles(&gcache,skalx,skaly,blackwhite);
	draw_gdk_base(&gcache,skalx,skaly,gc);
	draw_gdk_verticals(&gcache,skalx,skaly,blackwhite);
	for(j=(drawsingle ? curr_eq_get() : 0);j<(drawsingle ? curr_eq_get()+1 : MAX_EQUATIONS );++j){
		if(!pc[j].ptrtree)continue;
		if(pc[j].par.num>1 && pc[j].par_t[0].is){
			bool draw=true;
			for(i=1;i<pc[j].par.num-1;++i){
				pc[j].par.values[i]=pc[j].par_t[i-1].begin;
				if(pc[j].par_t[i].intvl==0){draw=false;break;}
			}
			if(draw)
				do{draw_gdk_graph(&gcache,skalx,skaly,pc+j,blackwhite);}while(!bf_algo_draw(pc+j));

		}
		else draw_gdk_graph(&gcache,skalx,skaly,pc+j,blackwhite);
	}	
	draw_gdk_points(&gcache,skalx,skaly,blackwhite);
	pixmap_to_graph();
	return TRUE;
}


gboolean entry_check_valvar(GtkWidget *w[],int n,bool *error){
	int j;
	GdkColor red={0,65535,0,0};
	for(j=0;j<n;++j){
		const char *endptr=NULL,*txt=gtk_entry_get_text((GtkEntry*)w[j]);
		gtk_widget_modify_bg(w[j],GTK_STATE_NORMAL,NULL);
		if(!txt){	*error=true;continue;}
		(void)strtod(gtk_entry_get_text(GTK_ENTRY(w[j])),(char**)&endptr);
		if(endptr && strlen(endptr)>0){
			gtk_widget_modify_bg(w[j],GTK_STATE_NORMAL,&red);
			*error=true;
		}
	}
	
}
/*
static gboolean on_schar_change(void){
	int i,j;
	bool error=false;
	if(!pc->ptrtree)return TRUE;
	for (i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i)
		entry_check_valvar(p2entry[i],3,&error);
	if(!error){
		for(i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i)
			if(pc[curr_eq].par.num>=i+2){
				pc[curr_eq].par_t[i].is=true;
				pc[curr_eq].par_t[i].begin=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][0])));
				pc[curr_eq].par_t[i].end=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][1])));
				pc[curr_eq].par_t[i].intvl=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][2])));
			}
			else pc->par_t[i].is=false;

	}
	if(GTK_IS_WIDGET(graph))REDRAW_GRAPH;
	return TRUE;
}

static gboolean on_drawrange_change(GtkWidget *w,GtkWidget *e[2]){
	int i;
	bool error=false;
	entry_check_valvar(e,2,&error);
	if(!error){
		pc[curr_eq].drawbordersx[0]=atof(gtk_entry_get_text((GtkEntry*)e[0]));
		pc[curr_eq].drawbordersx[1]=atof(gtk_entry_get_text((GtkEntry*)e[1]));
		if(pc[curr_eq].drawbordersx[0]>pc[curr_eq].drawbordersx[1]){
			double tmp=pc[curr_eq].drawbordersx[0];
			pc[curr_eq].drawbordersx[0]=pc[curr_eq].drawbordersx[1];
			pc[curr_eq].drawbordersx[1]=tmp;
		}
	}
	return TRUE;
}

static void on_singleval_change(void){
	struct result res;
	int i;
	const char* tmp=NULL;
	GdkColor red={0,65535,0,0};

	if(!pc[curr_eq].ptrtree) return;
	
	for(i=0;i<MAX_PARAMS && i<pc[curr_eq].par.num;++i){
		tmp=gtk_entry_get_text(GTK_ENTRY(entry1[i]));
		char *endptr=NULL;
		(void)strtod(tmp,&endptr);
		if (endptr && strlen(endptr)>0){
			gtk_widget_modify_bg(entry1[i],GTK_STATE_NORMAL,&red);
		}
		else{
			gtk_widget_modify_bg(entry1[i],GTK_STATE_NORMAL,NULL);
			singlevals[curr_eq][i]=pc[curr_eq].par.values[i]=exec_simplestring(tmp,0);		
		}
	}

	res=auswerten(pc[curr_eq].ptrtree,&pc[curr_eq].par);
	tmp=(res.defined ? dbltocstr(res.value,20) : strdup(labels_lang->res_undef));
	gtk_label_set_text(GTK_LABEL(rlabel),tmp);
	if(pc[curr_eq].par.num>1)valuetable_set();
	free(tmp);
}

static void param_entries(void){
	int i;
	char errtxt[40], tmp[2]="";
	const char *fnstr;
	parser_context *lpc=pc+curr_eq;
	
	free(lpc->par.values);
	lpc->par.values=NULL;
	lpc->par.num=0;
	subfn_free(lpc->ptrtree);
	free(lpc->ptrtree);

	fnstr=gtk_entry_get_text(GTK_ENTRY(fentry));
	gtk_label_set_text(GTK_LABEL(errlabel),"");

	free(lpc->fn_string);
	if(curr_eq!=MAX_EQUATIONS-1)
		lpc->fn_string=strdup(fnstr);
	else{/* prepend "f=" for calc mode * /
		lpc->fn_string=(char*)g_malloc(strlen(fnstr)+strlen("f=")+1);
		sprintf(lpc->fn_string,"f=%s",fnstr);
	}
	
	for(i=0;i<MAX_PARAMS;++i){
		gtk_widget_hide(entry1[i]);
		gtk_widget_hide(label1[i]);
		if(i>0)gtk_widget_hide(box[i-1]);
	}

	lpc->ptrtree=(subfunction*)calloc(1,sizeof(subfunction));
	if((i=einlesen(
	lpc->ptrtree,&lpc->par,lpc->fn_string)
	)!=0){
		snprintf(errtxt,40,labels_lang->parse_error,i);
		gtk_label_set_text(GTK_LABEL(errlabel),errtxt);
		free(lpc->ptrtree);lpc->ptrtree=NULL;
		if(GTK_IS_WIDGET(graph))REDRAW_GRAPH;
		return;
	}
	
	for(i=0;i<MAX_PARAMS;++i)
	if(lpc->par.num>=i+1){
		tmp[0]=pc->par.names[i];
		gtk_label_set_text(GTK_LABEL(label1[i]),tmp);
		gtk_widget_show(entry1[i]);
		gtk_widget_show(label1[i]);
	}
	for(i=2;i<=lpc->par.num && i<=MAX_PARAMS;++i)
		gtk_widget_show(box[i-2]);
	if(pc->par.num>1)main_window();/*schar label change * /
	on_singleval_change();
	//on_schar_change();
	if(GTK_IS_WIDGET(graph))REDRAW_GRAPH;
}
*/
static int saveini(const char* filename,const parser_context* pc){
	int i,j;
	FILE *abc=fopen(filename,"w");
	char *oldlocale=setlocale(LC_NUMERIC,NULL);
	setlocale(LC_NUMERIC,"C");/*decimal point*/
	fputs("[Andrea's Funktionsrechner-0.5]\n",abc);
	for (i=0;i<MAX_EQUATIONS;++i){
		if(!pc[i].ptrtree) continue;
		fprintf(abc,"str%i=%s\n",i,pc[i].fn_string);
		fprintf(abc,"sin%i=",i);
		for (j=0;j<pc[i].par.num;++j)
			fprintf(abc,"%f;",pc[i].par.values[j]);
		fprintf(abc,"\n");
		fprintf(abc,"grp%i=%f;%f;%f\n",i,pc[i].par_t[0].begin,pc[i].par_t[0].end,pc[i].par_t[0].intvl);
		fprintf(abc,"col%i=%i;%i;%i\n",i,pc[i].col.red,pc[i].col.green,pc[i].col.blue);
		fprintf(abc,"lwd%i=%i\n",i,pc[i].linewidth);
		fprintf(abc,"dbx%i=%f;%f\n",i,pc[i].drawbordersx[0],pc[i].drawbordersx[1]);
		fprintf(abc,"lin%i=%s;%d\n",i,pc[i].linestyle==line ? "line" : "dashed",pc[i].dashlen);
		fprintf(abc,"civ%i=%f\n",i,pc[i].calcintvl);
	}
	fprintf(abc,"dbw=%s\n",blackwhite ? "true" : "false");
	fprintf(abc,"box=%s\n",gc.boxes ? "true" : "false");
	fprintf(abc,"drs=%s\n",drawsingle ? "true" : "false");
	fprintf(abc,"lwd=%i\n",gc.linewidth);
	fprintf(abc,"skx=%i\n",skalx);
	fprintf(abc,"sky=%i\n",skaly);
	fprintf(abc,"zpr=%f;%f\n",zeropoint_rel.x,zeropoint_rel.y);
	fprintf(abc,"ceq=%i\n",curr_eq_get());

	if(gc.label_xaxis)	fprintf(abc,"lbx=%s\n",gc.label_xaxis);
	if(gc.label_yaxis)	fprintf(abc,"lby=%s\n",gc.label_yaxis);

	fprintf(abc,dump_misc());

	fclose(abc);
	setlocale(LC_NUMERIC,oldlocale);
}

static uint16_t openini_dbl_list_parse(const char *str,double* *out/* out=NULL by caller*/){
	short int num=0,n;
	double temp=0;
	while(sscanf(str,"%lf%hn%*c%hn",&temp,&n,&n)>0){
		str+=n;
		*out=(double*)g_realloc(*out,++num*sizeof(double));
		(*out)[num-1]=temp;
	}
	return num;
}
#ifdef USE_MGD2INI
void mgd2ini(char* mgd,char* ini);
void mgdinfo(char*);
static int openmgd(const char *filename){
#ifndef L_tmpnam
#define L_tmpnam 200
#endif
	char tmp[L_tmpnam];
	char *ininame="/tmp/afr-05rc1pre-1.ini";/*man pages an libc info say tmpnam's quite unsafe but I need a filename, not a FILE* pointer - temporary solution*/
	mgd2ini(filename,ininame);
	openini(ininame);
}
#endif /*USE_MGD2INI*/

static int openini(const char *filename){
	int i,j,npoint=0,nvert=0,narea=0,ncircle=0;
	char readin1[200],*readin2,*oldlocale;
	FILE *abc=fopen(filename,"r");
	oldlocale=setlocale(LC_NUMERIC,NULL);/*decimal point*/
	setlocale(LC_NUMERIC,"C");
	fgets(readin1,200,abc);
	if(strcmp(readin1,"[Andrea's Funktionsrechner-0.5]\n"))
		return -1;
	while (!feof(abc)){
		fgets(readin1,200,abc);
		if (!readin1) break;
		if (readin1[strlen(readin1)-1]=='\n') readin1[strlen(readin1)-1]=0;
		readin2=strpbrk(readin1,"=");
		if(!readin2 || !*readin2) break;
		*readin2++='\0';
		if(strlen(readin1)>=4 && isdigit(readin1[3])){
			uint16_t coeff=atoi(readin1+3);
			if (!strncmp("str",readin1,3)){
				free(pc[coeff].fn_string);
				pc[coeff].fn_string=strdup(readin2);
				if(coeff==curr_eq_get())
					fentry_read_pc();
				else{
					subfn_free(pc[coeff].ptrtree);
					free(pc[coeff].ptrtree);
					pc[coeff].par.num=0;
					free(pc[coeff].par.values);
					pc[coeff].par.values=NULL;
					pc[coeff].ptrtree=calloc(1,sizeof(subfunction));
					einlesen(pc[coeff].ptrtree,&pc[coeff].par,pc[coeff].fn_string);
				}
			}
			else if(!strncmp("lin",readin1,3)){
				if(strncmp("line",readin2,4)){
					pc[coeff].linestyle=line;
					pc[coeff].dashlen=atoi(readin2+5);
				}
				else if(strncmp("dashed",readin2,6)){
					pc[coeff].linestyle=dashed;
					pc[coeff].dashlen=atoi(readin2+7);
				}
			}
			else if(!strncmp("civ",readin1,3)){
				pc[coeff].calcintvl=MIN(fabs(atof(readin2)),1000);
			}
			else{
				double *tmp=NULL;
				int coeff=atoi(readin1+3),n;
				if (coeff<0 || coeff>=MAX_EQUATIONS) continue;
				n=openini_dbl_list_parse(readin2,&tmp);
				
				if(!strncmp("sin",readin1,3)){
					for (i=0;i<MAX_PARAMS && i<n;++i)
						singlevals[coeff][i]=tmp[i];
				}
				else if(!strncmp("grp",readin1,3)){
					double *temp=&pc[coeff].par_t[0].begin;
					for(i=0;i<3 && i<n;++i,++temp)
						*temp=tmp[i];
				}
				else if(!strncmp("col",readin1,3)){
					uint16_t *temp[3]={
						&pc[coeff].col.red,
						&pc[coeff].col.green,
						&pc[coeff].col.blue
					};
					for(i=0;i<n && i<3;++i)
						*temp[i]=tmp[i];
				}
				else if(!strncmp("dbx",readin1,3)){
					for(i=0;i<n && i<2;++i)
						pc[coeff].drawbordersx[i]=tmp[i];
					if(curr_eq_get()==coeff)
						main_window();
				}
				if(coeff=curr_eq_get())
					change_eq_nr(NULL,NULL);
				free(tmp);
			}
		}
		else if(!strcmp("dbw",readin1)){
			if(!strcmp("true",readin2))
				blackwhite=true;
			else if(!strcmp("false",readin2))
				blackwhite=false;
		}	
		else if(!strcmp("box",readin1)){
			if(!strcmp("true",readin2))
				gc.boxes=true;
			else if(!strcmp("false",readin2))
				gc.boxes=false;
			if(visible.options)optiondialog();
		}
		else if(!strcmp("drs",readin1)){
			if (!strcmp("true",readin2))
				drawsingle=true;
			else if(!strcmp("false",readin2))
				drawsingle=false;
			if(visible.options)optiondialog();
		}
		else if(!strcmp("lwd",readin1))
			gc.linewidth=strtod(readin2,NULL);
		else if(!strncmp("sk",readin1,2)){
			if(readin1[2]=='x')skalx=atoi(readin2);
			if(readin1[2]=='y')skaly=atoi(readin2);
			show_gwindow();
		}
		else if(!strcmp("ceq",readin1)){
			unsigned int ceq=atoi(readin2);
			if(ceq<MAX_PARAMS){ 
				curr_eq_set(ceq);
				change_eq_nr(NULL,NULL);
			}
		}
		else if(!strcmp("zpr",readin1)){
			double *d=NULL;
			int n=openini_dbl_list_parse(readin2,&d);
			if(n>=1)zeropoint_rel.x=d[0];
			if(n>=2)zeropoint_rel.y=d[1];
		}
		else if(!strcmp("lbx",readin1)){
			gc.label_xaxis=strdup(readin2);
			if(visible.options)optiondialog();
		}
		else if(!strcmp("lby",readin1)){
			gc.label_yaxis=strdup(readin2);
			optiondialog();
		}
		else if(!strcmp("POINT",readin1)){
			read_point(readin2,npoint++);
		}
		else if(!strcmp("VERT",readin1)){
			read_vertical(readin2,nvert++);
		}
		else if(!strcmp("AREA",readin1)){
			read_area(readin2,narea++);
		}
		else if(!strcmp("CIRCLE",readin1)){
			read_circle(readin2,ncircle++);
		}
	}
	REDRAW_GRAPH;
	fclose(abc);
	setlocale(LC_NUMERIC,oldlocale);
}

/*
static gboolean funcbuttonclick(GtkButton *b,gpointer in){
	int pos=gtk_editable_get_position(GTK_EDITABLE(fentry));
	gtk_editable_delete_selection(GTK_EDITABLE(fentry));
	gtk_editable_insert_text(GTK_EDITABLE(fentry),(char*)in,strlen((char*)in),&pos);
	gtk_editable_set_position(GTK_EDITABLE(fentry),pos);
}

static void funcbuttonfield(void){
	int i;
	static GtkWidget *win,*table,*button[38];
	static const char items[38][7]={
		"sin","cos","tan","sinh","cosh","tanh",
		"arcsin","arccos","arctan",
		"sqrt","ln","lg","exp",
		"round","floor","ceil","abs",
		"min","max",";",
		"1","2","3","4","5","6","7","8","9","0",
		"+","-","*","/","%","^","(",")"
	};
	static int instances;
	if(instances>0){
		gtk_widget_show_all(win);
		gtk_window_set_title(GTK_WINDOW(win),labels_lang->cap_funcbutton);
		
		return;
	}
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(win),10);
	gtk_window_set_title(GTK_WINDOW(win),labels_lang->cap_funcbutton);
	table=gtk_table_new(6,6,TRUE);
	gtk_container_add(GTK_CONTAINER(win),table);

	for (i=0;i<38;++i){
		button[i]=gtk_button_new_with_label(items[i]);
		g_signal_connect(G_OBJECT(button[i]),"clicked",G_CALLBACK(funcbuttonclick),(void*)items[i]);
		gtk_table_attach_defaults(GTK_TABLE(table),button[i],i%7,i%7+1,i/7,i/7+1);
	}
	g_signal_connect(G_OBJECT(win),"delete-event",G_CALLBACK(gtk_widget_hide_on_delete),NULL);
	gtk_widget_show_all(win);
	++instances;
}
*/


void parse_cmdline(int argc, char* argv[]){
	size_t i;
	for(i=1;i<argc;++i){
		if (strcmp(argv[i],"--help")==0)
			printf("Andrea's Funktionsrechner 0.5 beta\n"
				"%s [--help | --lang=LANG] [file(s)]\n"
				"LANG: [DE || EN || FR ]\n",argv[0]);
		else if(strncmp(argv[i],"--lang=",7)==0)
			if(strcmp(argv[i]+7,"DE")==0)
				labels_lang=&DE;
			else if(strcmp(argv[i]+7,"EN")==0)
				labels_lang=&EN;
			else if(strcmp(argv[i]+7,"FR")==0)
				labels_lang=&FR;
			else{
				printf("Wrong lang code. Exiting.\n");
				exit(1);
			}
		else if(file_exists(argv[i])) openini(argv[i]);
	}
}
void reset_all(void){
	skalx=skaly=50;
	curr_eq_set(0);
	gc.axis=gc.axis_scal=true;
	gc.linewidth=3;
	gc.box_intvl=0.5;
	gc.label_xaxis=gc.label_yaxis=NULL;
	gc.tracepoint=false;
	gc.tp_intvl=0.001;
	zeropoint_rel.x=zeropoint_rel.y=0.5;
	
	for(int i=0;i<MAX_EQUATIONS;++i){
		pc[i].linestyle=line;
		pc[i].dashlen=10;
	}
	show_gwindow();
	main_window();
	mainwin_accels_add(gwindow);
}
int main(int argc, char* argv[]){
	int i;
	gtk_init(&argc,&argv);

#ifdef __WIN32__
	char *l=g_win32_getlocale();
#else
	char *l=setlocale(LC_ALL,NULL);
#endif
	if(!strncmp("de",l,2))		labels_lang=&DE;
	else if(!strncmp("fr",l,2))	labels_lang=&FR;
	else				labels_lang=&EN;
	
	pc=(parser_context*)calloc(MAX_EQUATIONS,sizeof(parser_context));
	for  (i=0;i<MAX_EQUATIONS;++i){
		pc[i].linewidth=3;
		pc[i].drawbordersx[0]=-HUGE_VAL;
		pc[i].drawbordersx[1]=HUGE_VAL;
	}
	visible.points=visible.verticals=visible.options=false;
	
	
	reset_all();
	gcache.pm=NULL;
	valuetable_init();
	//funcbuttonfield();//done by changelang
	changelang(NULL);
	
	parse_cmdline(argc,argv);
	gtk_main();
}


/*Main window functions/callbacks* /
static gboolean lstyle_change(GtkToggleButton *c){
	pc[curr_eq].linestyle=gtk_toggle_button_get_active(c) ? dashed : line;
	REDRAW_GRAPH;
	return TRUE;
}
static gboolean cbutton_chcol(GtkWidget *w){
	gtk_color_button_get_color(GTK_COLOR_BUTTON(w),&pc[curr_eq].col);
	REDRAW_GRAPH;
	return TRUE;
}
static gboolean calcintvl_change(GtkSpinButton *sb){
	pc[curr_eq].calcintvl=gtk_spin_button_get_value(sb);
	REDRAW_GRAPH;
	return TRUE;
}

static gboolean ival(GtkSpinButton *b,int num){
	integr_vals[curr_eq][num-1]=gtk_spin_button_get_value(b);
	return FALSE;
}
static gboolean iqad1(GtkSpinButton *w,GtkLabel *r){
	double de,a;
	char *x;
	de=MIN(integr_vals[curr_eq][0],integr_vals[curr_eq][1]);
	a=MAX(integr_vals[curr_eq][0],integr_vals[curr_eq][1]);
	if(pc[curr_eq].ptrtree)
		x=dbltocstr(integrate_qad(pc[curr_eq].ptrtree,&pc[curr_eq].par,de,a)*(de<=a ? 1 : -1),12);
	gtk_label_set_text(r,pc[curr_eq].ptrtree ? x  : labels_lang->res_undef);
	return TRUE;
}
static gboolean wskal_change(GtkSpinButton *sb){
	pc[curr_eq].linewidth=gtk_spin_button_get_value_as_int(sb);
	REDRAW_GRAPH;
	return FALSE;
}
static void main_window(void){
	static GtkWidget *window,*button[2], *flabel,*table[4],*vpan,*ntab,*ntab_label[3],*widthskal,*widthlab,*widthbox,*drawbordersb,*drawbordersl[2],*drawborderse[2],*lstyle,
		*menualign,*menu, *menu_main_itemlist[4],*menu_file_items[4][6],*menu_main[4];
	static bool exists;
	int i,j;
	char tmp[50];
	static gboolean (*menu_funcptr[4][8])(void)={
		{new_wrapper,open_wrapper,save_wrapper,vtab_export_wrapper,graph_export_wrapper,print_wrapper,exit_wrapper},
		{graph_points,graph_verticals,vector_stuff,graph_areas,graph_circles,shell_window},
		{valuetable_init,show_gwindow_full,optiondialog,NULL,NULL,NULL,NULL,NULL},
		{help_wrapper,aboutbox,NULL,NULL,NULL,NULL,NULL,NULL},
	};
	static int menu_item_no[]={8,7,4,3};/*incl. captions!! * /

	if(exists){
		for(i=0;i<3;++i)
			gtk_label_set_text(GTK_LABEL(ntab_label[i]),labels_lang->cap_tabs[i]);
		for(i=0;i<4;++i){
			gtk_label_set_text_with_mnemonic(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menu_main[i]))),labels_lang->menu_labels[i][0]);
			for(j=1;j<menu_item_no[i];++j){
				gtk_label_set_text_with_mnemonic(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menu_file_items[i][j-1]))),labels_lang->menu_labels[i][j]);
				if(!isspace(labels_lang->menu_accels[i][j-1])){
					snprintf(tmp,50,"<AFRMain>/Menu%d/%d",i,j);
					gtk_accel_map_change_entry(tmp,labels_lang->menu_accels[i][j-1],GDK_CONTROL_MASK,TRUE);
				}
			}
		}
		for(i=1;i<=MAX_EQUATIONS-1;++i){
			gtk_combo_box_remove_text(GTK_COMBO_BOX(flabel),i-1);
			snprintf(tmp,15,"%s %d:",labels_lang->flabel_norm,i);
			gtk_combo_box_insert_text(GTK_COMBO_BOX(flabel),i-1,tmp);
		}
		gtk_combo_box_remove_text(GTK_COMBO_BOX(flabel),i-1);
		gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),labels_lang->flabel_calculator);
		gtk_combo_box_set_active(GTK_COMBO_BOX(flabel),curr_eq);

		for(i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i){
			char tmp[strlen(labels_lang->schar_entry)];
			sprintf(tmp,labels_lang->schar_entry,pc[curr_eq].par.names[i+1]);
			gtk_label_set_text((GtkLabel*)label2[i],tmp);
		}
		gtk_spin_button_set_value((GtkSpinButton*)widthskal,pc[curr_eq].linewidth);
		return;
	}

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 10);
	gtk_window_set_title((GtkWindow*)window, "Andrea's Funktionsrechner 0.5beta4 (GTK2)");
	
	vpan=gtk_vpaned_new();
	for(i=0;i<4;++i){
		table[i]=gtk_table_new(7,3,FALSE);
		gtk_table_set_row_spacings(GTK_TABLE(table[i]),5);
		gtk_table_set_col_spacings(GTK_TABLE(table[i]),5);
	}
	ntab=gtk_notebook_new();
	for (i=0;i<3;++i){
		ntab_label[i]=gtk_label_new(labels_lang->cap_tabs[i]);
		gtk_notebook_append_page(GTK_NOTEBOOK(ntab),table[i+1],ntab_label[i]);
	}
	gtk_paned_pack1(GTK_PANED(vpan),table[0],FALSE,FALSE);
	gtk_paned_pack2(GTK_PANED(vpan),ntab,FALSE,FALSE);
	gtk_widget_show(ntab);
	
	gtk_container_add(GTK_CONTAINER(window), vpan);
	
	/*begin menu* /
	menualign=gtk_alignment_new(0,0,0,0);
	gtk_table_attach_defaults(GTK_TABLE(table[0]),menualign,0,3,0,1);

	menu=gtk_menu_bar_new();
	gtk_container_add(GTK_CONTAINER(menualign),menu);

	for(i=0;i<4;++i){
		menu_main_itemlist[i]=gtk_menu_new();
		menu_accel[i]=gtk_accel_group_new();
		for (j=1;j<menu_item_no[i];++j){
			snprintf(tmp,50,"<AFRMain>/Menu%d/%d",i,j);
			gtk_menu_set_accel_group((GtkMenu*)menu_main_itemlist[i],menu_accel[i]);
			menu_file_items[i][j-1]=gtk_image_menu_item_new_with_mnemonic(labels_lang->menu_labels[i][j]);
			gtk_widget_set_accel_path(menu_file_items[i][j-1],tmp,menu_accel[i]);
			if(!isspace(labels_lang->menu_accels[i][j-1]))
				gtk_accel_map_add_entry(tmp,labels_lang->menu_accels[i][j-1],GDK_CONTROL_MASK);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu_main_itemlist[i]),menu_file_items[i][j-1]);
			if(menu_funcptr[i][j-1])
				g_signal_connect(G_OBJECT(menu_file_items[i][j-1]),"activate",G_CALLBACK(menu_funcptr[i][j-1]),NULL);
		}

		menu_main[i]=gtk_menu_item_new_with_mnemonic(labels_lang->menu_labels[i][0]);
		gtk_menu_set_accel_group((GtkMenu*)menu_main_itemlist[i],menu_accel[i]);
		gtk_window_add_accel_group((GtkWindow*)window,menu_accel[i]);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_main[i]);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_main[i]),menu_main_itemlist[i]);
	}

	/*ugly but type-saving macro* /
#define menu_stock_image(I,J,STOCK)\
	gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_file_items[I][J],gtk_image_new_from_stock(STOCK,GTK_ICON_SIZE_MENU))
	
	menu_stock_image(0,0,GTK_STOCK_NEW);
	menu_stock_image(0,1,GTK_STOCK_OPEN);
	menu_stock_image(0,2,GTK_STOCK_SAVE_AS);
	menu_stock_image(0,5,GTK_STOCK_PRINT);
	menu_stock_image(0,6,GTK_STOCK_QUIT);
	menu_stock_image(2,2,GTK_STOCK_PREFERENCES);
#undef menu_stock_image
	/*end menu* /
	
	errlabel=gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table[0]), errlabel, 0,3, 2,3);
	
	flabel=gtk_combo_box_new_text();
	for (i=1;i<=MAX_EQUATIONS-1;++i){
		snprintf(tmp,15,"%s %d:",labels_lang->flabel_norm,i);
		gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),tmp);
	}
	gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),labels_lang->flabel_calculator);
	gtk_combo_box_set_active(GTK_COMBO_BOX(flabel),curr_eq);
	g_signal_connect(G_OBJECT(flabel),"changed",G_CALLBACK(change_eq_nr),(void*)table[1]);
	gtk_table_attach_defaults(GTK_TABLE(table[0]),flabel,0,1, 4,5);
	
	fentry=gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(table[0]),fentry,2,3, 4,5);
	g_signal_connect(G_OBJECT(fentry),"changed",G_CALLBACK(param_entries),NULL);

	rlabel=gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table[2]), rlabel,0,3, 0,1);

	for(i=0;i<MAX_PARAMS;++i){
		entry1[i]=gtk_entry_new();
		gtk_table_attach_defaults(GTK_TABLE(table[2]),entry1[i],2,3,2+2*i,3+2*i);
		gtk_entry_set_text(GTK_ENTRY(entry1[i]),"0");
		g_signal_connect(G_OBJECT(entry1[i]),"changed",G_CALLBACK(on_singleval_change),NULL);
		gtk_widget_set_no_show_all(entry1[i],TRUE);
		label1[i]=gtk_label_new("");
		gtk_table_attach_defaults(GTK_TABLE(table[2]),label1[i],0,1,2+2*i,3+2*i);
	}
	
	/* tab "drawing": * /
	colorbutton=gtk_color_button_new();
	gtk_color_button_set_title(GTK_COLOR_BUTTON(colorbutton),labels_lang->color);
	gtk_table_attach_defaults(GTK_TABLE(table[1]), colorbutton, 0,1 ,1,2);
	g_signal_connect(G_OBJECT(colorbutton),"notify::color",G_CALLBACK(cbutton_chcol),NULL);

	lstyle=gtk_check_button_new_with_label("- - - -");
	gtk_table_attach_defaults((GtkTable*)table[1],lstyle, 1,2, 1,2);
	g_signal_connect((GObject*)lstyle,"clicked",(GCallback)lstyle_change,NULL);

	widthbox=gtk_vbox_new(FALSE,5);
	widthlab=gtk_label_new_with_mnemonic(labels_lang->linewidth);
	widthskal=gtk_spin_button_new_with_range(1,50,1);
	g_signal_connect((GObject*)widthskal,"value-changed",(GCallback)wskal_change,NULL);
	gtk_label_set_mnemonic_widget((GtkLabel*)widthlab,widthskal);
	gtk_box_pack_start_defaults((GtkBox*)widthbox,widthlab);
	gtk_box_pack_start_defaults((GtkBox*)widthbox,widthskal);
	gtk_table_attach_defaults((GtkTable*)table[1],widthbox,2,3, 1,2);

	drawbordersb=gtk_hbox_new(FALSE,4);
	drawbordersl[0]=gtk_label_new("Zeichnen von ");
	drawbordersl[1]=gtk_label_new("bis ");
	for (i=0;i<2;++i){
		drawborderse[i]=gtk_entry_new();
		g_signal_connect((GObject*)drawborderse[i],"changed",(GCallback)on_drawrange_change,(void*)drawborderse);
		gtk_box_pack_start_defaults((GtkBox*)drawbordersb,drawbordersl[i]);
		gtk_box_pack_start_defaults((GtkBox*)drawbordersb,drawborderse[i]);
	}
	gtk_entry_set_text((GtkEntry*)drawborderse[0],"-100000");
	gtk_entry_set_text((GtkEntry*)drawborderse[1],"100000");
	gtk_table_attach_defaults((GtkTable*)table[1],drawbordersb,0,3, 3,4);
	g_signal_connect((GObject*)flabel,"changed",(GCallback)dborders_eq,(void*)drawborderse);
	
	struct{GtkWidget *box,*lab1,*intvl,*lab2;}calcintvl;
	calcintvl.box=gtk_hbox_new(FALSE,3);
	calcintvl.lab1=gtk_label_new("Zeichenpunkte nur ausrechnen alle");
	calcintvl.intvl=gtk_spin_button_new_with_range(0.000,1000,0.001);
	calcintvl.lab2=gtk_label_new("Einheiten (0=alle ausrechnen)");
	g_signal_connect((GObject*)calcintvl.intvl,"value-changed",(GCallback)calcintvl_change,NULL);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.lab1);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.intvl);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.lab2);
	gtk_table_attach_defaults((GtkTable*)table[1],calcintvl.box,0,3, 4,5);
	

	for(i=0;i<MAX_PARAMS-1;++i){
		box[i]=gtk_hbox_new(FALSE,5);
		gtk_widget_set_no_show_all(box[i],TRUE);
		label2[i]=gtk_label_new(labels_lang->schar_entry);
		gtk_box_pack_start_defaults(GTK_BOX(box[i]),label2[i]);
		gtk_widget_show(label2[i]);
		for (j=0;j<3;++j){
			p2entry[i][j]=gtk_entry_new();
			gtk_box_pack_start_defaults(GTK_BOX(box[i]),p2entry[i][j]);
			g_signal_connect(G_OBJECT(p2entry[i][j]),"changed",G_CALLBACK(on_schar_change),NULL);
			gtk_widget_show(p2entry[i][j]);
		}
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][0]),"-1");
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][1]),"1");
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][2]),"1");
		gtk_table_attach_defaults(GTK_TABLE(table[1]),box[i],0,3,6+i,7+i);
	}
	
	/*iqad(integration) stuff:* /
	GtkWidget *ent[2],*res=gtk_label_new(""),
		  *lab1=gtk_label_new("von"),
		  *lab2=gtk_label_new("bis");
	ent[0]=gtk_spin_button_new_with_range(-1000,1000,0.1);
	ent[1]=gtk_spin_button_new_with_range(-1000,1000,0.1);
	gtk_table_attach_defaults((GtkTable*)table[3],res,0,3,0,1);
	gtk_table_attach_defaults((GtkTable*)table[3],lab1,0,1,2,3);
	gtk_table_attach_defaults((GtkTable*)table[3],ent[0],2,3,2,3);
	gtk_table_attach_defaults((GtkTable*)table[3],lab2,0,1,4,5);
	gtk_table_attach_defaults((GtkTable*)table[3],ent[1],2,3,4,5);

	g_signal_connect((GObject*)ent[0],"value-changed",G_CALLBACK(ival),(void*)1);
	g_signal_connect((GObject*)ent[1],"value-changed",G_CALLBACK(ival),(void*)2);

	g_signal_connect((GObject*)ent[0],"value-changed",G_CALLBACK(iqad1),res);
	g_signal_connect((GObject*)ent[1],"value-changed",G_CALLBACK(iqad1),res);
	g_signal_connect((GObject*)fentry,"changed",G_CALLBACK(iqad1),res);
	
	
	g_signal_connect(G_OBJECT(window),"delete-event",G_CALLBACK(exit_wrapper),NULL);
	gtk_widget_show_all(window);
	exists=true;
}


/*Graph window functions/callbacks:*/
static gboolean skalset(GtkSpinButton *in,int* var){
	*var=gtk_spin_button_get_value_as_int(in);
	REDRAW_GRAPH;
	return TRUE;
}

void graph_to_clipboard(void){
	GtkClipboard *clip=gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
	GdkPixbuf *pb=gdk_pixbuf_get_from_drawable(NULL,gcache.pm,
		gdk_colormap_get_system(),0,0,0,0,gcache.width,gcache.height);
	gtk_clipboard_set_image(clip,pb);
}
static gboolean showgraphpos(GtkWidget *w,GdkEventMotion *pos,gpointer poslabel){
	char help[40];
	static double x,y;
	int curr_eq=curr_eq_get();
	struct {int x,y;}ursprung={zero_x(),zero_y()};
	if(gc.tracepoint && pc[curr_eq_get()].ptrtree){/*tracepoint is drawn on graph and not in gcache to be able to delete tracepoint fast*/
		GdkGC *gcon=gdk_gc_new(graph->window);
		int rad=pc[curr_eq].linewidth*5;
		gdk_draw_drawable(graph->window,gcon,gcache.pm, (int)x-rad,(int)y-rad, (int)x-rad,(int)y-rad, 2*rad,2*rad);/*delete old tracepoint*/
		x=round(x_gdk_reverse(pos->x)/gc.tp_intvl)*gc.tp_intvl; /*round current xposition to n*tp_intvl , x used otherwise*/
		if(pc[curr_eq].par.num>0)pc[curr_eq].par.values[0]=x;
		struct result res=auswerten(pc[curr_eq].ptrtree,&pc[curr_eq].par);
		y=res.defined ? res.value : HUGE_VAL;
		snprintf(help,40,"P(%9.3f|%9.3f) T:%5.3f",x,y,gc.tp_intvl);
		gdk_draw_arc(graph->window,gcon,TRUE,x_gdk(x)-rad/2,y_gdk(y)-rad/2,rad,rad,0,360*64);/* arc defined by top left edge, not middle ;( */ 
		x=x_gdk(x);y=y_gdk(y);
		//free gcon!
	}
	else snprintf(help,40,"P(%9.3f|%9.3f)",x_gdk_reverse(pos->x),y_gdk_reverse(pos->y));
	gtk_label_set_text(GTK_LABEL(poslabel),help);
	return TRUE;
}
void show_gwindow_full(void){
	gtk_widget_show_all(gwindow);
	show_gwindow();
}
static int show_gwindow(void){
	static GtkWidget *vpan,*propbox,*skal[2]/*x,y*/,*label,*poslabel;
	static int instances;
	int i;
	static GdkColor white={0,65535,65535,65535};

	GtkAccelGroup *ac;
	GClosure *func;
	
	if(instances>0){
		gtk_label_set_text(GTK_LABEL(label),labels_lang->axis_scale);
		gtk_spin_button_set_value((GtkSpinButton*)skal[0],skalx);
		gtk_spin_button_set_value((GtkSpinButton*)skal[1],skaly);
		
		return TRUE;
	}
	propbox=gtk_hbox_new(FALSE,5);

	for (i=0;i<2;++i){
		skal[i]=gtk_spin_button_new_with_range(5,100000,1);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(skal[i]),i<1 ? skalx : skaly);
		g_signal_connect(G_OBJECT(skal[i]),"value-changed",G_CALLBACK(skalset),(void*)(i<1 ? &skalx : &skaly));
	}
	
	gtk_box_pack_start_defaults(GTK_BOX(propbox),skal[0]);
	
	label=gtk_label_new(labels_lang->axis_scale);
	gtk_box_pack_start_defaults(GTK_BOX(propbox),label);
	
	gtk_box_pack_start_defaults(GTK_BOX(propbox),skal[1]);
	
	poslabel=gtk_label_new("");
	gtk_box_pack_start_defaults(GTK_BOX(propbox),poslabel);

	graph=gtk_drawing_area_new();
	gtk_widget_modify_bg(graph,GTK_STATE_NORMAL,&white);
	gtk_widget_set_size_request(graph,100,100);
	gtk_widget_add_events(graph,GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK);
	g_signal_connect(G_OBJECT(graph),"expose_event",G_CALLBACK(pixmap_to_graph),NULL);
	g_signal_connect(G_OBJECT(graph),"button-press-event",G_CALLBACK(graph_move),NULL);
	g_signal_connect(G_OBJECT(graph),"button-release-event",G_CALLBACK(graph_move),NULL);
	g_signal_connect(G_OBJECT(graph),"motion-notify-event",G_CALLBACK(showgraphpos),(void*)poslabel);

	gwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	vpan=gtk_vpaned_new();
	gtk_paned_pack1(GTK_PANED(vpan),propbox,TRUE,TRUE);
	gtk_paned_pack2(GTK_PANED(vpan),graph,TRUE,TRUE);
	gtk_paned_set_position(GTK_PANED(vpan),50);

	gtk_container_add(GTK_CONTAINER(gwindow),vpan);
	gtk_window_resize(GTK_WINDOW(gwindow),800,600);
	g_signal_connect(G_OBJECT(gwindow),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),(void*)gwindow);
	gtk_widget_show_all(gwindow);

	/*Let the user copy the shown graph via clipboard:*/
	ac=gtk_accel_group_new();
	func=g_cclosure_new((GCallback)graph_to_clipboard,NULL,NULL);
	gtk_accel_group_connect(ac,'c',GDK_CONTROL_MASK,0,func);
	gtk_window_add_accel_group((GtkWindow*)gwindow,ac);

	instances++;
	return TRUE;
}

gboolean graph_move(GtkWidget *widget,GdkEventButton *event){
	zeropoint_rel.x=((float)event->x)/graph->allocation.width;
	zeropoint_rel.y=((float)event->y)/graph->allocation.height;
	REDRAW_GRAPH;
	return FALSE;
}

/*Valuetable stuff:*/
gboolean valuetable_init(void){
	static GtkTreeViewColumn *column[18];
	static GtkWidget *win;
	static int instances;
	char *tmp;
	float i;

	if(instances>0){
		gtk_widget_show_all(win);
		gtk_window_set_title(GTK_WINDOW(win),labels_lang->cap_valuetable);
		return TRUE;
	}
	GtkCellRenderer *rend=gtk_cell_renderer_text_new();
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(win), 10);
	listst=gtk_list_store_new(17,
	G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
	G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
	gtk_list_store_append(listst,&iter);
	for(i=-4;i<=4;i+=0.5){
		gtk_list_store_set(listst,&iter,(int)((i+4)*2),"",-1);
	}
	valuetable=gtk_tree_view_new_with_model(GTK_TREE_MODEL(listst));

	gtk_container_add(GTK_CONTAINER(win), valuetable);
	g_signal_connect(G_OBJECT(win),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),(void*)win);
	gtk_widget_show_all(win);
	for (i=-4;i<=4;i+=0.5){
		int x=(i+4)*2;
		tmp=dbltocstr(i,5);
		column[x]=gtk_tree_view_column_new_with_attributes(tmp,rend,"text",x,NULL);
		free(tmp);
		gtk_tree_view_append_column(GTK_TREE_VIEW(valuetable),column[x]);
	}
	g_object_unref(rend);
	instances++;
	valuetable_init();
	return TRUE;
}

gboolean valuetable_set(void){
	float i;
	float start,end,intvl;
	uint16_t curr_eq=curr_eq_get();
	char *enter;
	struct result res={0,false,false};
	if (pc[curr_eq].par.num>=1){
		/*
start=round(x_gdk_reverse(0)/box_line_intvl)*box_line_intvl; //auf vielfache von box_line_intvl runden
end=round(x_gdk_reverse(size.width)/box_line_intvl)*box_line_intvl;
intvl=((end-start)/17)
		
		*/
		for (i=-4;i<=4;i+=0.5){
			pc[curr_eq].par.values[0]=i;
			if(pc[curr_eq].ptrtree)res=auswerten(pc[curr_eq].ptrtree,&pc[curr_eq].par);
			if (res.defined)
				enter=dbltocstr(res.value,10);
			else{
				enter=(char*)g_malloc(7);
				strcpy(enter,"n.def.");
			}
			gtk_list_store_set(listst,&iter,(int)((i+4)*2),enter,-1);
			free(enter);
		}
	}
	return TRUE;
}
static gboolean valuetable_export(char *filename){
	/* CSV export of current vtab */
	double x;
	uint16_t curr_eq=curr_eq_get();
	FILE *abc=fopen(filename,"w");
	char *oldlocale=setlocale(LC_NUMERIC,NULL);
	setlocale(LC_NUMERIC,"C");
	if(!abc)return FALSE;
	fprintf(abc,"%c,\"%s\"\n",pc[curr_eq].par.num>0 ? pc[curr_eq].par.names[0] : ' ',pc[curr_eq].fn_string);
	for (x=-4;x<=4;x+=0.5){
		if(pc[curr_eq].par.num<1) break;
		pc[curr_eq].par.values[0]=x;
		struct result res=auswerten(pc[curr_eq].ptrtree,&pc[curr_eq].par);
		fprintf(abc,"%f,%f\n",x,(res.defined && isfinite(res.value) ? res.value : NAN));
	}
	fclose(abc);
	setlocale(LC_NUMERIC,oldlocale);
	return TRUE;
}

/*General stuff for changing of current equation: * /
gboolean change_eq_nr(GtkComboBox *widget, GtkWidget *in){
	int i;
	char *tmp;
	if(widget)curr_eq=gtk_combo_box_get_active(widget);
	if(curr_eq>MAX_EQUATIONS) curr_eq=0;
	gtk_entry_set_text(GTK_ENTRY(fentry),(pc[curr_eq].fn_string ? pc[curr_eq].fn_string+(curr_eq==MAX_EQUATIONS-1 ? 2 : 0) : ""));
	for(i=0;i<pc[curr_eq].par.num && i<MAX_PARAMS;++i){
		tmp=dbltocstr(singlevals[curr_eq][i],100);
		gtk_entry_set_text(GTK_ENTRY(entry1[i]),tmp);
		free(tmp);
	}
	//specifics for calc mode
	if (curr_eq==MAX_EQUATIONS-1){
		if (in) gtk_widget_hide(in); /*dont show graph or single calc stuff * /
		gtk_widget_hide(gwindow);//
	}
	else{
		if(in) gtk_widget_show(in);
		gtk_widget_show(gwindow);//
		gtk_color_button_set_color(GTK_COLOR_BUTTON(colorbutton),&pc[curr_eq].col);
		REDRAW_GRAPH;
	}
	on_singleval_change();
	if(curr_eq!=MAX_EQUATIONS-1){
		valuetable_set();
		for (i=0;i<pc[curr_eq].par.num-1;++i)
		if(pc[curr_eq].par_t[i].is){
			char temp[20];
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].begin);
			gtk_entry_set_text((GtkEntry*)p2entry[i][0],temp);
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].end);
			gtk_entry_set_text((GtkEntry*)p2entry[i][1],temp);
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].intvl);
			gtk_entry_set_text((GtkEntry*)p2entry[i][2],temp);
		}
	}
	return FALSE;
}
static gboolean dborders_eq(GtkWidget *w,GtkEntry *e[2]){
	char x[40];
	snprintf(x,40,"%f",pc[curr_eq].drawbordersx[0]);
	gtk_entry_set_text(e[0],x);
	snprintf(x,40,"%f",pc[curr_eq].drawbordersx[1]);
	gtk_entry_set_text(e[1],x);
	REDRAW_GRAPH;
	return TRUE;
}
*/

gboolean print_wrapper(void){
	size_t i,j;
	if(curr_eq_get()==MAX_EQUATIONS-1){
		ShowErrorDlg("No printing in calculator mode available");
		return TRUE;
	}
#ifdef USE_GTK2_PRINT
	cairo_pc cpc={pc,drawsingle,blackwhite,gc,zeropoint_rel,skalx,skaly};
	if(drawsingle) cpc.pc+=curr_eq_get();
#ifdef OLDPRINT
	GtkWidget *savedialog=gtk_file_chooser_dialog_new("Save Postscript File",NULL,GTK_FILE_CHOOSER_ACTION_SAVE,GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_SAVE,GTK_RESPONSE_ACCEPT,NULL);
	if(gtk_dialog_run(GTK_DIALOG(savedialog))==GTK_RESPONSE_ACCEPT){
		const char *tmp=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(savedialog));
		if(file_exists_dialog(tmp)){
			cpc.filename=tmp;
			print_gtk2(&cpc);
		}
	}
	gtk_widget_destroy(savedialog);
#else
	print_gtk2(&cpc);
#endif/*OLDPRINT*/
#endif
	return TRUE;
}

/* Option dialog functions/callbacks:*/
gboolean change_check_opt(GtkToggleButton *tb,bool* prop){
	*prop=gtk_toggle_button_get_active(tb);
	REDRAW_GRAPH;
	return TRUE;
}
gboolean change_float_opt(GtkSpinButton *sb,float* prop){
	*prop=gtk_spin_button_get_value(sb);
	REDRAW_GRAPH;
	return TRUE;
}
gboolean change_float_opt2(GtkRange *r,float *prop){
	*prop=gtk_range_get_value(r);
	REDRAW_GRAPH;
	return TRUE;
}
gboolean change_str_opt(GtkEntry *e,char* prop[]){
	free(*prop);
	*prop=strdup(gtk_entry_get_text(e));
	REDRAW_GRAPH;
	return TRUE;
}
int changelang(GtkWidget *w){
	int i;
	if (w){ 
		i=gtk_combo_box_get_active(GTK_COMBO_BOX(w));
		labels_lang=labelhandle.l[i];
	}
	else{
		i=((size_t)labels_lang-(size_t)labelhandle.l[0])/sizeof(struct labels);/*gcc bug workaround*/
	}
#ifdef __WIN32_ /* temporary ugly hack */
	setlocale(LC_ALL,strncmp("de",langs_short[i],2) ? "English_United States" : "German_Germany");
#else
	setlocale(LC_ALL,langs_short[i]);
#endif
	/* refresh labels: */
	main_window();
	if(visible.options)			optiondialog();
	funcbuttonfield();
	if(visible.points)			graph_points();
	if(visible.verticals)			graph_verticals();
	if(visible.areas)			graph_areas();
	if(GTK_WIDGET_VISIBLE(gwindow))		show_gwindow();
	if(GTK_WIDGET_VISIBLE(valuetable))	valuetable_init();
}


#define odialog_add_opt(target,label,callback,cb_ptr,pack){\
	target=gtk_check_button_new_with_mnemonic(label);\
	if(callback)g_signal_connect((GObject*)target,"toggled",(GCallback)callback,(void*)cb_ptr);\
	if(pack)gtk_box_pack_start_defaults((GtkBox*)vbox,target);}


gboolean optiondialog(void){
	uint32_t i;
	ptrdiff_t j;
	static GtkWidget *win, *tab,*setwidgets[16],*vbox;
	static int instances;
	const bool *tmp[]={&gc.boxes,&drawsingle,&blackwhite,&gc.axis,&gc.axis_scal};
	static struct{GtkWidget *tab,*lx,*ly,*ex,*ey;}axislab;
	static struct{GtkWidget *hbox,*onoff,*intvl;}tracep;
	
	if(instances>0){
		gtk_widget_hide(win);
		gtk_widget_show_all(win);
		for(i=0;i<5;++i)
			gtk_label_set_text_with_mnemonic(GTK_LABEL(gtk_bin_get_child(GTK_BIN(setwidgets[i]))),labels_lang->options[i]);
		gtk_window_set_title(GTK_WINDOW(win),labels_lang->cap_options);
		/*get current property values (e.g. after openini()):*/
		for (i=0;i<5;++i)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(setwidgets[i]),*tmp[i]);
		gtk_label_set_text_with_mnemonic((GtkLabel*)setwidgets[9],labels_lang->linewidth);
		gtk_spin_button_set_value((GtkSpinButton*)setwidgets[10],gc.linewidth);
		gtk_label_set_text((GtkLabel*)axislab.lx,labels_lang->xlabel);
		gtk_entry_set_text((GtkEntry*)axislab.ex,gc.label_xaxis ? gc.label_xaxis : "");
		gtk_label_set_text((GtkLabel*)axislab.ly,labels_lang->ylabel);
		gtk_entry_set_text((GtkEntry*)axislab.ey,gc.label_yaxis ? gc.label_yaxis : "");
		gtk_range_set_value((GtkRange*)setwidgets[6],gc.box_intvl);
		return TRUE;
	}

	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	tab=gtk_table_new(8,3,TRUE);
	vbox=gtk_vbox_new(FALSE,10);
	gtk_container_add(GTK_CONTAINER(win),vbox);
	for (i=0;i<5;++i){
		setwidgets[i]=gtk_check_button_new_with_mnemonic(labels_lang->options[i]);
		g_signal_connect((GObject*)setwidgets[i],"toggled",(GCallback)change_check_opt,(void*)tmp[i]);
		if(i!=4)gtk_table_attach_defaults(GTK_TABLE(tab),setwidgets[i],0,1,i*2,i*2+1);
	}
	gtk_table_attach_defaults((GtkTable*)tab,setwidgets[4],1,2,6,7);
	//gtk_box_pack_start_defaults((GtkBox*)vbox,
	
	g_signal_connect((GObject*)setwidgets[3],"toggled",(GCallback)opt_subopt_active,(void*)setwidgets[4]);
	
	setwidgets[5]=gtk_font_button_new();
	gtk_table_attach_defaults((GtkTable*)tab,setwidgets[5],2,3,6,7);
	g_signal_connect((GObject*)setwidgets[5],"font-set",(GCallback)fontbutton_changed,(void*)&gc.ascal_font);
	g_signal_connect((GObject*)setwidgets[5],"font-set",(GCallback)redraw_graph,NULL);
		
	setwidgets[6]=gtk_hscale_new_with_range(0.1,2,0.1);
	g_signal_connect((GObject*)setwidgets[6],"value-changed",(GCallback)change_float_opt2,(void*)&gc.box_intvl);
	gtk_table_attach_defaults((GtkTable*)tab,setwidgets[6],1,2,0,1);
	
	setwidgets[7]=gtk_combo_box_new_text();
	for(i=0;i<langnum;++i)
		gtk_combo_box_append_text(GTK_COMBO_BOX(setwidgets[7]),langs[i]);
	j=((size_t)labels_lang-(size_t)labelhandle.l[0])/sizeof(struct labels);
	gtk_combo_box_set_active(GTK_COMBO_BOX(setwidgets[7]),j);
	gtk_table_attach_defaults(GTK_TABLE(tab),setwidgets[7],0,1,8,9);
	g_signal_connect(G_OBJECT(setwidgets[7]),"changed",G_CALLBACK(changelang),NULL);
	
	gtk_box_pack_start_defaults((GtkBox*)vbox,tab);
	
	setwidgets[8]=gtk_hbox_new(FALSE,2);
	setwidgets[9]=gtk_label_new(labels_lang->linewidth);//fuer Achsen/Kaestchen
	setwidgets[10]=gtk_spin_button_new_with_range(1,50,1);
	g_signal_connect((GObject*)setwidgets[10],"value-changed",(GCallback)change_int_opt,(void*)&gc.linewidth);
	gtk_label_set_mnemonic_widget((GtkLabel*)setwidgets[9],setwidgets[10]);
	gtk_box_pack_start_defaults((GtkBox*)setwidgets[8],setwidgets[9]);
	gtk_box_pack_start_defaults((GtkBox*)setwidgets[8],setwidgets[10]);
	//gtk_table_attach_defaults((GtkTable*)tab,setwidgets[8],0,1,8,9);
	gtk_box_pack_start_defaults((GtkBox*)vbox,setwidgets[8]);
	
	axislab.tab=gtk_table_new(2,2,TRUE);
	axislab.lx=gtk_label_new(labels_lang->xlabel);
	axislab.ex=gtk_entry_new();
	axislab.ly=gtk_label_new(labels_lang->ylabel);
	axislab.ey=gtk_entry_new();
	g_signal_connect((GObject*)axislab.ex,"changed",(GCallback)change_str_opt,(void*)&gc.label_xaxis);
	g_signal_connect((GObject*)axislab.ey,"changed",(GCallback)change_str_opt,(void*)&gc.label_yaxis);
	gtk_table_attach_defaults((GtkTable*)axislab.tab,axislab.lx,0,1,0,1);
	gtk_table_attach_defaults((GtkTable*)axislab.tab,axislab.ex,1,2,0,1);
	gtk_table_attach_defaults((GtkTable*)axislab.tab,axislab.ly,0,1,1,2);
	gtk_table_attach_defaults((GtkTable*)axislab.tab,axislab.ey,1,2,1,2);
	gtk_box_pack_start_defaults((GtkBox*)vbox,axislab.tab);
	
	tracep.hbox=gtk_hbox_new(FALSE,3);
	tracep.onoff=gtk_check_button_new_with_mnemonic("Mit Y-Koordinate aktivem Graph folgen mit Intervall");
	tracep.intvl=gtk_spin_button_new_with_range(0.001,100,0.001);
	g_signal_connect((GObject*)tracep.onoff,"toggled",(GCallback)change_check_opt,(void*)&gc.tracepoint);
	g_signal_connect((GObject*)tracep.intvl,"value-changed",(GCallback)change_float_opt,(void*)&gc.tp_intvl);
	g_signal_connect((GObject*)tracep.onoff,"toggled",(GCallback)opt_subopt_active,(void*)tracep.intvl);
	gtk_box_pack_start_defaults((GtkBox*)tracep.hbox,tracep.onoff);
	gtk_box_pack_start_defaults((GtkBox*)tracep.hbox,tracep.intvl);
	gtk_box_pack_start_defaults((GtkBox*)vbox,tracep.hbox);
	
	
	
	visible.options=true;
	g_signal_connect(G_OBJECT(win),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),(void*)win);
	g_signal_connect((GObject*)win,"delete_event",(GCallback)bool_set_false,(void*)&visible.options);
	g_signal_connect((GObject*)win,"delete_event",(GCallback)gtk_widget_hide_on_delete,NULL);	
	++instances;
	optiondialog();
	return TRUE;
}

static gboolean shell_exec(void* null,GtkEntry *e){
	const char *command=gtk_entry_get_text(e);
	if(strcmp(command,"info")==0) ShowErrorDlg("Sorry, but the shell is not yet ready for use...");
	else if(strcmp(command,"hypothesis_stuff")==0) hypothesis_stuff();
	else if(strcmp(command,"export_png")==0){
		int w,h;
		gdk_drawable_get_size(graph->window,&w,&h);
		GdkPixbuf *pb=gdk_pixbuf_get_from_drawable(NULL,graph->window,
				gdk_colormap_get_system(),0,0,0,0,w,h);
		gdk_pixbuf_save(pb,"andrea_fkt.png","png",NULL,"compression","9",NULL);
	}
}
gboolean shell_window(void){
	GtkWidget *win,*tab,*inp,*outp,*but;
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	tab=gtk_table_new(FALSE,5,4);
	gtk_container_add((GtkContainer*)win,tab);
	inp=gtk_entry_new();
	outp=gtk_label_new("work in progress");
	but=gtk_button_new_with_mnemonic("_Go!");
	gtk_table_attach_defaults((GtkTable*)tab,inp,0,3,0,1);
	gtk_table_attach_defaults((GtkTable*)tab,outp,0,5,2,4);
	gtk_table_attach_defaults((GtkTable*)tab,but,4,5,0,1);
	g_signal_connect((GObject*)but,"clicked",(GCallback)shell_exec,(void*)inp);
	gtk_widget_show_all(win);
}
