/* math_parser_gtk2.h */
#ifndef MATH_PARSER_GTK2_H
#define MATH_PARSER_GTK2_H MATH_PARSER_GTK2_H
#include <gtk/gtk.h>
#include "math_parser.h"
#include "math_parser_compat.h"
#define MAX_EQUATIONS 20 /* last one reserved for calculator mode */
#define MAX_PARAMS 3

struct schar{
	bool is;
	double begin,end,intvl;
};

enum{line,dashed,pointed,none};

typedef struct {
	subfunction *ptrtree;
	struct params par;
	struct schar par_t[MAX_PARAMS-1];
	char* fn_string;
	GdkColor col, col2;
	uint16_t linewidth;char linestyle;uint16_t dashlen;
	double drawbordersx[2];
	float calcintvl;/* 0/negative for 'normal' constant drawing, otherwise calculating one point per intvl */
	bool draw_jumps;/*whether jumps as in round or modulo functions should be drawn (i.e with vertical lines)*/
}parser_context;

typedef struct{/*global, i.e. for basic drawing*/
	bool boxes,axis,axis_scal;
	float box_intvl;
	char boxstyle,axisstyle;
	char *ascal_font;
	uint16_t linewidth;
	char *label_xaxis,*label_yaxis;
	char *lfont_xaxis,*lfont_yaxis;
	bool tracepoint;
	float tp_intvl;
}graph_context;

typedef struct{
	GdkPixmap *pm;
	int width,height;
	PangoLayout *pango;
}graph_cache;

enum drawtype{DT_GDK,DT_PS,DT_CAIRO};

char* dbltocstr(double,int x);
char* inttocstr(int i,int x);

uint16_t curr_eq_get(void);
int zero_x(void);
int zero_y(void);

void redraw_graph(void);

gboolean entry_check_valvar(GtkWidget *w[],int n,bool *error);

struct xy_float{
	float x,y;
};

#ifdef USE_GTK2_PRINT
typedef struct{
	parser_context *pc;
	bool drawsingle,blackwhite;
	graph_context gc;
	struct xy_float zerop_rel;
	int skalx,skaly;
#ifdef OLDPRINT
	const char *filename;
#endif /*OLDPRINT*/
}cairo_pc;
#endif /*USE_GTK2_PRINT*/

#ifdef MAIN_WIN_INCLUDING
gboolean optiondialog(void);
gboolean shell_window(void);
gboolean valuetable_init(void);
gboolean valuetable_set(void);
void show_gwindow_full(void);
gboolean vtab_export_wrapper(void);
gboolean graph_export_wrapper(void);

gboolean new_wrapper(void);
gboolean open_wrapper(void);
gboolean save_wrapper(void);
gboolean print_wrapper(void);
gboolean exit_wrapper(void);
gboolean help_wrapper(void);
#endif

#endif /*MATH_PARSER_GTK2_H*/
