/*math_parser_gtk2_draw.c*/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "math_parser_gtk2.h"
#include "math_parser_gtk2_mainwindow.h"
#include "math_parser_gtk2_draw.h"
#include "math_parser_gtk2_misc.h"
#include "math_parser_gtk2_helperfns.h"

static float box_line_intvl(enum drawtype dt,float skal){
/* Determining the box/scale intvl depending on the scalation; every 2nd skal can have a label*/
static int i;
static float tab[][2]={
	{100000,0.00025},{50000, 0.0005},{25000, 0.001},{15000,0.0025},	{9000, 0.005},
	{3200, 0.01},	{1600, 0.02},	{900, 0.025}, /* %.2f format for graph scale label */
	{500, 0.05},	{160, 0.1},
	{80, 0.25}, /* %.1f */
	{40, 0.5},	{25, 1.0},	{15, 2.0},	{10, 5.0},	{6, 10.0},	{5,20.0}
};
	for(i=0;i<sizeof(tab)/sizeof(float)/2;++i)
		if(skal>=tab[i][0]) return tab[i][1];
}
static char* box_line_labelf(int skal){
	static const char *f[4]={"%.0f","%.1f","%.2f","%.3f"};
	if(skal>=15000)	return f[3];
	if(skal>=900) 	return f[2];
	if(skal>=80)	return f[1];
	return f[0];	
}

static bool sfn_jump(subfunction *jsf,size_t jnum,struct result *jres[3],const struct params par){
	/* jnum : num of sfns in jsf(contains each jump possible func extra)
	 * jres[0..2] must be malloc'ed(jnum*sizeof()) by the caller
	 * jres[new/old/oldold][jsf[]]
	 * Designed to work when res.spruenge (i.e. when function contains % or round-like things)
	 * Do not call the function for the first point to be drawn ;)*/
	size_t i;
	for(i=0;i<jnum;++i){
		jres[2][i].value=jres[1][i].value;
		jres[1][i].value=jres[0][i].value;
		jres[0][i]=subfn_evaluate(&jsf[i],&par);
		if(jsf[i].type!='m'){
			if(fabs(jres[0][i].value-jres[1][i].value)>=DBL_EPSILON)
				return true;
		}/*else if =='m'*/
		else if(jres[1][i].value-jres[2][i].value>0 && jres[0][i].value-jres[1][i].value<0) return true;
		else if(jres[1][i].value-jres[2][i].value<0 && jres[0][i].value-jres[1][i].value>0) return true;	
	}
	return false;
}

static bool asymptotic_jump(const struct result *res,const struct result *oldres,const struct result *oldoldres){
return ((res->value-oldres->value<0 != oldres->value-oldoldres->value<0)&&(fabs(3*(oldres->value-oldoldres->value))<fabs((res->value-oldres->value))|| fabs(oldres->value-oldoldres->value)<3*fabs(res->value-oldres->value)));
}

gboolean draw_gdk_base(graph_cache *gcache,int skalx,int skaly,graph_context gcon){
	double i;
	double drawx,drawy;/* determines skal[xy] multiplicator */
	GtkRequisition size={gcache->width,gcache->height};
	GdkGC *gc;
	GdkColor /*abc={777,24400,2000,1700},*/black={888,0,0,0},white={0,65535,65535,65535};
	struct {int x,y;} ursprung={zero_x(),zero_y()};
	GdkDrawable *da=gcache->pm;
	char skallabel[10];
	PangoLayout *pangolayout=pango_layout_copy(gcache->pango);
	pango_layout_set_alignment(pangolayout,PANGO_ALIGN_RIGHT);
	if(gcon.ascal_font)
		PangoLayout_setfont_str(pangolayout,gcon.ascal_font);
	gc=gdk_gc_new(da);
	gdk_gc_set_rgb_fg_color(gc,&black);

	drawx=box_line_intvl(DT_GDK,skalx);
	drawy=box_line_intvl(DT_GDK,skaly);

	if(gcon.boxes){
		float f=MAX(gcon.box_intvl*2,0.2);
		for(i=y_gdk(ceil(y_gdk_reverse(0)/drawy/f)*drawy*f);i<size.height;i+=skaly*drawy*f){
			if(gcon.box_intvl==0)
				gdk_gc_set_line_attributes(gc,fmod(y_gdk_reverse(i),0.5)==0 ? 1 : 0.5,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
			gdk_draw_line(da,gc,0,i,size.width,i);
			for(int n=0;n<=size.width/10;)
			gdk_draw_line(da,gc,10*++n,i,10*++n,i);
		}
		for(i=x_gdk(ceil(x_gdk_reverse(0)/drawx/f)*drawx*f);i<size.width;i+=skalx*drawx*f){
			gdk_draw_line(da,gc,i,0,i,size.height);
		}
	}
	
	gdk_gc_set_line_attributes(gc,gcon.linewidth,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);

	/* Achsen */
	if(gcon.axis){
		gdk_draw_line(da,gc,0,y_gdk(0),size.width,y_gdk(0));
		gdk_draw_line(da,gc,x_gdk(0),0,x_gdk(0),size.height);
		
		if(gcon.axis_scal){
	/* Skalen : ermittelt virtuelle koord des Aussenrandes,rundet auf naechste virtuelle ganzahlige koordinate auf,rechnet dies in GtkDrawing(echte)koord um*/
			for (i=y_gdk(ceil(y_gdk_reverse(0)/drawy)*drawy);i<size.height;i+=skaly*drawy){
				gdk_draw_line(da,gc,x_gdk(0)+2,i,x_gdk(0)-2,i);
				if(gcon.axis_scal && y_gdk_reverse(i) && fmod(y_gdk_reverse(i),drawy*2)==0){
					char* abc=box_line_labelf(skaly);
					snprintf(skallabel,10,abc,y_gdk_reverse(i));
					pango_layout_set_text(pangolayout,skallabel,-1);
					gdk_draw_layout(da,gc,x_gdk(0)+5,i-5,pangolayout);
				}
			}
			for (i=x_gdk(ceil(x_gdk_reverse(0)/drawx)*drawx);i<size.width;i+=skalx*drawx){
				gdk_draw_line(da,gc,i,y_gdk(0)+2,i,y_gdk(0)-2);
				if(gcon.axis_scal && x_gdk_reverse(i) && fmod(x_gdk_reverse(i),drawx*2)==0){
					char* abc=box_line_labelf(skalx);
					snprintf(skallabel,10,abc,x_gdk_reverse(i));
					pango_layout_set_text(pangolayout,skallabel,-1);
					gdk_draw_layout(da,gc,i-strlen(skallabel)*3,y_gdk(0)+2,pangolayout);
				}
			}
			/*Pfeilspitzen,Achsenbeschriftung*/
			gdk_draw_line(da,gc,x_gdk(0),0,x_gdk(0)+4,5);
			gdk_draw_line(da,gc,x_gdk(0),0,x_gdk(0)-4,5);
			if(gcon.label_yaxis){
				pango_layout_set_text(pangolayout,gcon.label_yaxis,-1);
				gdk_draw_layout(da,gc,x_gdk(0)+5,5,pangolayout);
			}
			gdk_draw_line(da,gc,size.width,y_gdk(0),size.width-5,y_gdk(0)-4);
			gdk_draw_line(da,gc,size.width,y_gdk(0),size.width-5,y_gdk(0)+4);
			if(gcon.label_xaxis){
				pango_layout_set_text(pangolayout,gcon.label_xaxis,-1);
				gdk_draw_layout(da,gc,size.width-strlen(gcon.label_xaxis)*5,y_gdk(0)+8,pangolayout);
			}
		}
	}
	return TRUE;
}

static void linedraw_gdk_dashed(GdkDrawable * da, GdkGC *gc,const int from_x,const int from_y,const int to_x,const int to_y){
	int dashlen=10;
	extern parser_context *pc;
	static bool used,draw_on;
	static int pxcount,dlencount;/*pxcount: px waiting to be (not) drawn, dlencount indicates how many px to !draw_on*/
	int tmp1_x,tmp1_y,tmp2_x,tmp2_y;
	if(used) return; /* cheap locking mechanism */
	used=true;
	
	dashlen=pc[curr_eq_get()].dashlen;
	
	pxcount=(int)sqrt(pow(to_y-from_y,2)+pow(to_x-from_x,2));
	if(dlencount==0){dlencount=dashlen;draw_on=!draw_on;}
	assert(dlencount>0);
	
	tmp1_x=from_x;
	tmp1_y=from_y;
	while(pxcount>0){
		tmp2_x=MAX(tmp1_x+1,tmp1_x+(to_x-tmp1_x)*MIN(dlencount/((double)pxcount),1));/* shorten line (at end) to remaining pixels*/
 		tmp2_y=tmp1_y+(to_y-tmp1_y)*MIN(dlencount/((double)pxcount),1);/* for current drawing state, if necessary*/
		if(draw_on)gdk_draw_line(da,gc,tmp1_x,tmp1_y,tmp2_x,tmp2_y);
		pxcount-=sqrt(pow(tmp2_y-tmp1_y,2)+pow(tmp2_x-tmp1_x,2));/*Phytagoras*/
		dlencount-=sqrt(pow(tmp2_y-tmp1_y,2)+pow(tmp2_x-tmp1_x,2));
		if(dlencount<=0){dlencount=dashlen;draw_on=!draw_on;/*change draw status*/}
		tmp1_x=tmp2_x;
		tmp1_y=tmp2_y;
	}
	used=false;
	return;
}


gboolean draw_gdk_graph(graph_cache *gcache,int skalx,int skaly,parser_context* pc,bool grayscale){
	int i,j,jnum=0,dashcount=0;
	bool drawing_dash=true;
	void (*linedraw)(GdkDrawable*,GdkGC*,int,int,int,int)=pc->linestyle==dashed ? linedraw_gdk_dashed : gdk_draw_line;
	double x,calcstep,calcstart,calcstop;
	bool oldposvalid;
	GtkRequisition size={gcache->width,gcache->height};
	GtkRequisition oldoldpos={-1,-1},oldpos={-1,-1},newpos;/* for lineto/moveto emulation */
	GdkColor black={888,0,0,0},white={0,65535,65535,65535};
	GdkDrawable *da=gcache->pm;
	GdkGC *gc;
	struct result res,oldres,oldoldres,*jres[3];
	struct {int x,y;} ursprung={zero_x(),zero_y()};
	gc=gdk_gc_new(da);
	if(pc->par.num==0 || pc->par.values==NULL || pc->ptrtree==NULL) return FALSE;
	
	subfunction *sf=calloc(1,sizeof(subfunction));
	subfn_give_subfn_jump_fn_pars(&sf,pc->ptrtree,&jnum);
	for(i=0;i<=2;++i)
		jres[i]=(struct result*)g_malloc(jnum*sizeof(struct result));

	gdk_gc_set_rgb_fg_color(gc,grayscale ? &black : &pc->col);
	gdk_gc_set_line_attributes(gc,pc->linewidth,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
	
	if(pc->calcintvl<=0){
		calcstep=1.0/MAX(skalx,50);
		calcstart=MAX(x_gdk_reverse(0),pc->drawbordersx[0]);
		calcstop=MIN(x_gdk_reverse(size.width),pc->drawbordersx[1]);
		pc->draw_jumps=false;/*see below */
	}
	else{/*calculate in intvls on*/
		calcstep=pc->calcintvl;
		calcstart=calcstep*floor(MAX(x_gdk_reverse(0),pc->drawbordersx[0])/calcstep);
		/*untere Grenze calcstart soll Vielfaches von calcstep sein*/
		calcstop=MIN(x_gdk_reverse(size.width),pc->drawbordersx[1])+calcstep;
		pc->draw_jumps=true;/*debug*/
	}
	for (x=calcstart;x<=calcstop;x+=calcstep){
		oldposvalid=isfinite(res.value) && res.defined;
		pc->par.values[0]=x;
		res=auswerten(pc->ptrtree,&pc->par);
		newpos.width=x_gdk(x);
		newpos.height=y_gdk(res.value);
		if (res.defined && isfinite(res.value)/*C99*/){
			if (pc->draw_jumps) goto draw;
			/* check for asymptotes w/ sign change and jumps(->round...)*/
			if (newpos.height<0) newpos.height=-200;
			if (newpos.height>size.height)newpos.height=size.height+100;
			if(asymptotic_jump(&res,&oldres,&oldoldres));
				       /*||*/else if( (oldpos.height>size.height && newpos.height<0 )||(oldpos.height<=0 && newpos.height>size.height)|| (oldpos.width!=-1 && res.spruenge && sfn_jump(sf,jnum,jres,pc->par)));
			else
draw:			if (oldpos.width!=-1 && oldposvalid)linedraw(da,gc,oldpos.width,oldpos.height,newpos.width,newpos.height);
		}
savepos:	oldoldpos=oldpos;
		oldoldres=oldres;
		oldpos=newpos;
		oldres=res;
		if (oldpos.width>size.width) break;
	}
	free(jres[0]);
	free(jres[1]);
	for(i=jnum-1;i>=0;--i)
		subfn_free(&sf[i]);
	free(sf);
	g_object_unref(gc);
	return TRUE;
}

#include <stdio.h>
#include <locale.h>
#include "math_parser.h"

#ifdef USE_GTK2_PRINT
/* Requires GTK2 >= 2.10 w/ Pangocairo */

#ifdef OLDPRINT /* PS output (untested, works probably only in "C" locale) */

#define gtk_print_context_get_dpi_x(arg) (72.0)
#define gtk_print_context_get_dpi_y(arg) (72.0)

#define gtk_print_context_get_width(arg)  (20.0*cm_x) /* ca. DIN A4 */
#define gtk_print_context_get_height(arg) (28.0*cm_y)

#define cairo_t FILE
#define cairo_move_to(cr,x,y)	fprintf(cr,"%f %f moveto\n",x,y)
#define cairo_line_to(cr,x,y)	fprintf(cr,"%f %f lineto\n",x,y)
#define cairo_clip(cr)		fprintf(cr,"clip\n""newpath\n")
#define cairo_stroke(cr)	fprintf(cr,"stroke\n""newpath\n")
#define cairo_set_line_width(cr,f)	fprintf(cr,"%f setlinewidth\n",f)
#define cairo_set_source_rgb(cr,r,g,b)\
	fprintf(cr,"%f %f %f setrgbcolor\n",r,g,b)

#define y_cairo ((int)ursprung.y+(Y)*skaly)
#define y_cairo_reverse (((Y)-ursprung.y)/(double)skaly)

#define PangoLayout FILE
#define pango_cairo_create_layout(cr)		(cr)
#define PangoLayout_setfont_str(pl,str)\
	fprintf(pl,"/Times-Roman findfont\n""12 scalefont\n""setfont\n")
#define PangoLayout_show_cairo(cr,pl,str)\
	fprintf(cr,"(%s) show\n",str)

#endif /*OLDPRINT*/

static void draw_cairo(cairo_t *cr,int skalx,int skaly,parser_context *pc,struct root ursprung,GtkRequisition size,float cm_x,float cm_y){
	struct result res,oldres,oldoldres;
	struct params par=pc->par;
	double i,calcstep,calcstart,calcstop;
	GtkRequisition oldpos={-1,-1};
	
	/* jump detection (round,modulo) init:*/
	int jnum=0;
	struct result *jres[3];
	subfunction *jsf=calloc(1,sizeof(subfunction));
	subfn_give_subfn_jump_fn_pars(&jsf,pc->ptrtree,&jnum);
	for(i=0;i<=2;++i)
		jres[(int)i]=(struct result*)g_malloc(jnum*sizeof(struct result));
		
	if(pc->linestyle==dashed){
		double dlen[1]={pc->dashlen/10*0.4*cm_x};
		cairo_set_dash(cr,dlen,1,0);
	}

	if(pc->calcintvl<=0){
		calcstep=fabs(x_cairo_reverse(0)-x_cairo_reverse(1.0/72));
		calcstart=MAX(x_cairo_reverse(0),pc->drawbordersx[0]);
		calcstop=MIN(x_cairo_reverse(size.width),pc->drawbordersx[1]);
		pc->draw_jumps=false;/*see below */
	}
	else{/*calculate in intvls on*/
		calcstep=pc->calcintvl;
		calcstart=calcstep*floor(MAX(x_cairo_reverse(0),pc->drawbordersx[0])/calcstep);
		/*untere Grenze calcstart soll Vielfaches von calcstep sein*/
		calcstop=MIN(x_cairo_reverse(size.width),pc->drawbordersx[1])+calcstep;
		pc->draw_jumps=true;/*debug*/
	}
	for (i=calcstart;i<=calcstop;i+=calcstep){
		par.values[0]=i;
		res=auswerten(pc->ptrtree,&par);
		if (res.defined){
			if (oldpos.height==-1 || asymptotic_jump(&res,&oldres,&oldoldres) || (oldpos.height>size.height && y_cairo(res.value)<0) || (oldpos.height<0 && y_cairo(res.value)>size.height)
				|| (jres && res.spruenge && sfn_jump(jsf,jnum,jres,par)))
				cairo_move_to(cr,x_cairo(i),y_cairo(res.value));
			else cairo_line_to(cr,x_cairo(i),y_cairo(res.value));
		}
		oldpos.width=i;
		oldpos.height=y_cairo(res.value);
		oldoldres=oldres;
		oldres=res;
	}
	cairo_stroke(cr);
	cairo_set_dash(cr,NULL,0,0);
}
#ifndef OLDPRINT
static void PangoLayout_show_cairo(cairo_t *cr,PangoLayout *pl,const char str[]){
	pango_layout_set_text(pl,str,-1);
	pango_cairo_show_layout(cr,pl);
}
#endif
static void draw_page(GtkPrintOperation *operation,GtkPrintContext *context,int page_nr,cairo_pc *cpc){
        struct result res;
	double i,calcstep;
	int j;
        cairo_t *cr;
	GtkRequisition size,oldpos={-1,-1};
        float cm_x=gtk_print_context_get_dpi_x(context)/2.54;
        float cm_y=gtk_print_context_get_dpi_y(context)/2.54;
	float skalx=cm_x /box_line_intvl(DT_CAIRO,cpc->skalx);
	float skaly=cm_y /box_line_intvl(DT_CAIRO,cpc->skaly);
	
#ifndef OLDPRINT
        cr=gtk_print_context_get_cairo_context(context);
#else
	cr=fopen(cpc->filename,"w");
#endif
        size.width=gtk_print_context_get_width(context);
        size.height=gtk_print_context_get_height(context);
	struct root ursprung={cm_x*3+cm_x*(18-3)*cpc->zerop_rel.x,cm_y*8+cm_y*(20-8)*cpc->zerop_rel.y};
	
	PangoLayout *pl=gtk_print_context_create_pango_layout(context);
	PangoLayout_setfont_str(pl,"Sans 16");
	cairo_move_to(cr,cm_x*3,cm_y*1);
	PangoLayout_show_cairo(cr,pl,"Andis Funktionsrechner 0.5beta\n (C) Andreas Oster,2007,2008");
	cairo_stroke(cr);
	
	/*Legend*/
	PangoLayout_setfont_str(pl,"Sans 12");
	for(j=0;j<(cpc->drawsingle? 1 : MAX_EQUATIONS-1);++j)
		if(cpc->pc[j].fn_string && cpc->pc[j].ptrtree){
			cairo_move_to(cr,cm_x*7,cm_y*(3+j/2.0));
			cairo_set_source_rgb(cr,cpc->pc[j].col.red/65535.0,cpc->pc[j].col.green/65535.0,cpc->pc[j].col.blue/65535.0);
			PangoLayout_show_cairo(cr,pl,cpc->pc[j].fn_string);
		}
	cairo_stroke(cr);
	cairo_set_source_rgb(cr,0,0,0);
	
	/*Wertetabelle*/
	{
		parser_context* pc=cpc->drawsingle ? cpc->pc : &cpc->pc[curr_eq_get()];
		if(pc && pc->ptrtree){
			char abc[30];
			struct params par=pc->par;
			struct result res;

			PangoLayout_setfont_str(pl,"Sans 10");

			snprintf(abc,30,"Wertetabelle Funktion %d",curr_eq_get());
			cairo_move_to(cr,cm_x*7,cm_y*22);
			PangoLayout_show_cairo(cr,pl,abc);

			for(i=-4;i<=4;i+=0.5){
				snprintf(abc,10,"%2.2f",i);
				cairo_move_to(cr,cm_x*(5+(i+2)*2.5),cm_y*23);
				PangoLayout_show_cairo(cr,pl,abc);
				if(par.num>0)par.values[0]=i;
				res=subfn_evaluate(pc->ptrtree,&par);
				snprintf(abc,10,"%2.2f",res.defined ? res.value : NAN);
				cairo_move_to(cr,cm_x*(5+(i+2)*2.5),cm_y*24);
				PangoLayout_show_cairo(cr,pl,abc);
			}
			cairo_stroke(cr);
		}
	}
	/*Boundingbox*/
	cairo_move_to(cr,cm_x*3,cm_y*8);
	cairo_line_to(cr,cm_x*18,cm_y*8);
	cairo_line_to(cr,cm_x*18,cm_y*20);
	cairo_line_to(cr,cm_x*3,cm_y*20);
	cairo_line_to(cr,cm_x*3,cm_y*8);
	cairo_clip(cr);
	
	/*Kaestchen*/
	if(cpc->gc.boxes){
/* cm_y/skaly==box_line_intvl(DT_CAIRO,cpc->skaly) in the for-loop
 * used to round to a (integer) multiple of the (end-user visible)scalation interval box_line_intvl(...) at the top of the document
 * without it drawing may start at y=0 for box_line_intvl(...)<<1)
 * same for x
 */
		cairo_set_line_width(cr,0.2);
		for(i=y_cairo(round(y_cairo_reverse(0)/(cm_y/skaly)) *cm_y/skaly);i<size.height;i+=cm_y/2){
			cairo_move_to(cr,0,i);
			cairo_line_to(cr,size.width,i);
		}
		for (i=x_cairo(round(x_cairo_reverse(0)*(cm_x/skalx)) *(skalx/cm_x));i<size.width;i+=cm_x/2){
			cairo_move_to(cr,i,0);
			cairo_line_to(cr,i,size.height);
		}
		cairo_stroke(cr);
	}
	cairo_set_line_width(cr,1);
#ifndef OLDPRINT
	draw_cairo_areas(cr,skalx,skaly,ursprung,cpc->blackwhite,cpc->pc);
#endif

	/* Achsen */
	if(cpc->gc.axis){
		cairo_set_source_rgb(cr,0,0,0);
		cairo_move_to(cr,x_cairo(0),0);
		cairo_line_to(cr,x_cairo(0),size.height);
		cairo_move_to(cr,0,y_cairo(0));
		cairo_line_to(cr,size.width,y_cairo(0));
	
	/* Skalen */
		if(cpc->gc.axis_scal){
			PangoLayout_setfont_str(pl,cpc->gc.ascal_font ? cpc->gc.ascal_font : "Sans 8");
			for (i=y_cairo(ceil(y_cairo_reverse(0)));i<size.height;i+=skaly*box_line_intvl(DT_CAIRO,cpc->skaly)){
				cairo_move_to(cr,x_cairo(0)+3,i);
				cairo_line_to(cr,x_cairo(0)-3,i);
			
				cairo_move_to(cr,x_cairo(0)-8,i);
				if(y_cairo_reverse(i)!=0 && fmod(round_2(y_cairo_reverse(i),5),box_line_intvl(DT_CAIRO,cpc->skaly)*2)==0){
					char skallabel[10];
					snprintf(skallabel,10,box_line_labelf(skaly),y_cairo_reverse(i));
					PangoLayout_show_cairo(cr,pl,skallabel);
//fprintf(stderr, "%f\n" , fmod(y_cairo_reverse(i),box_line_intvl(DT_CAIRO,cpc->skaly)));
				}
			}
			for (i=x_cairo(ceil(x_cairo_reverse(0)));i<size.width;i+=skalx*box_line_intvl(DT_CAIRO,cpc->skalx)){
				cairo_move_to(cr,i,y_cairo(0)+3);
				cairo_line_to(cr,i,y_cairo(0)-3);
			
				cairo_move_to(cr,i,y_cairo(0)+5);
				if(x_cairo_reverse(i)!=0 && fmod(x_cairo_reverse(i),box_line_intvl(DT_CAIRO,cpc->skalx)*2)==0){
					char skallabel[10];
					snprintf(skallabel,10,box_line_labelf(skalx),x_cairo_reverse(i));
					PangoLayout_show_cairo(cr,pl,skallabel);
				}
			}
			cairo_stroke(cr);

			/*Pfeilspitzen und Achsenbeschriftung*/
			pango_layout_set_alignment(pl,PANGO_ALIGN_RIGHT);

			cairo_move_to(cr,x_cairo(0)+4,cm_y*8);
			cairo_line_to(cr,x_cairo(0),cm_y*8-5);
			cairo_line_to(cr,x_cairo(0)-4,cm_y*8);
			PangoLayout_show_cairo(cr,pl,cpc->gc.label_yaxis);

			cairo_move_to(cr,cm_x*18-5,y_cairo(0)-4);
			cairo_line_to(cr,cm_x*18,y_cairo(0));
			cairo_line_to(cr,cm_x*18-5,y_cairo(0)+4);
			PangoLayout_show_cairo(cr,pl,cpc->gc.label_xaxis);
		}
		cairo_stroke(cr);
	}
#ifndef OLDPRINT
	draw_cairo_points(cr,skalx,skaly,ursprung,cpc->blackwhite);
	draw_cairo_verticals(cr,skalx,skaly,ursprung,cpc->blackwhite);
	draw_cairo_areas(cr,skalx,skaly,ursprung,cpc->blackwhite);
	draw_cairo_circles(cr,skalx,skaly,ursprung,cpc->blackwhite);
#endif
	
	for(j=0;j<(cpc->drawsingle? 1 : MAX_EQUATIONS-1);++j)
		if(cpc->pc[j].ptrtree){
			int i;
			if(!cpc->blackwhite)
				cairo_set_source_rgb(cr,cpc->pc[j].col.red/65535.0,cpc->pc[j].col.green/65535.0,cpc->pc[j].col.blue/65535.0);
			cairo_set_line_width(cr,cpc->pc[j].linewidth/3.0);
			if(cpc->pc[j].par.num>1){
				for(i=1;i<cpc->pc[j].par.num;++i){
					cpc->pc[j].par.values[i]=cpc->pc[j].par_t[i-1].begin;
					if(cpc->pc->par_t[i-1].intvl==0) continue;
				}
				do{draw_cairo(cr,skalx,skaly,cpc->pc+j,ursprung,size,cm_x,cm_y);}while(!bf_algo_draw(cpc->pc+j));
			}
			else draw_cairo(cr,skalx,skaly,cpc->pc+j,ursprung,size,cm_x,cm_y);
		}
}

gboolean print_gtk2(cairo_pc *cpc){
#ifndef OLDPRINT
	GtkPrintOperation *print;
	GtkPrintOperationResult res;
	static GtkPrintSettings *conf=NULL;
	GError *err=g_malloc(sizeof(GError));
	print=gtk_print_operation_new();
	if(conf)gtk_print_operation_set_print_settings(print,conf);
	else conf=gtk_print_settings_new();
	gtk_print_operation_set_n_pages(print,1);
	GtkPageSetup *page=gtk_print_run_page_setup_dialog(NULL,NULL,conf);
	gtk_print_operation_set_show_progress(print,TRUE);
	g_signal_connect(print,"draw_page",G_CALLBACK(draw_page),(void*)cpc);
	res = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,NULL, &err);
	if (res == GTK_PRINT_OPERATION_RESULT_APPLY){
		if (conf != NULL)
			g_object_unref (conf);
		conf = g_object_ref (gtk_print_operation_get_print_settings (print));
	}
	g_object_unref(print);
#else
	static char oldloc[20]=setlocale(LC_NUMERIC,NULL);
	setlocale(LC_NUMERIC,"C");
	draw_page(NULL,NULL,1,cpc);
	setlocale(LC_NUMERIC,oldloc);
#endif
}

#endif /* USE_GTK2_PRINT */
