/*main window
 *contains main window incl.funcbuttons, and aboutbox
 */
#define MAIN_WIN_INCLUDING

#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "math_parser_gtk2.h"
#include "math_parser_gtk2_helperfns.h"
#include "math_parser_gtk2_lang.h"
#include "math_parser_gtk2_misc.h"

static uint16_t curr_eq; /*which equation is current[0..MAX_EQUATIONS-1] */
uint16_t curr_eq_get(void){return curr_eq;}
uint16_t curr_eq_set(uint16_t in){return (in<MAX_EQUATIONS ? curr_eq=in : -1);}

static GtkWidget *box[MAX_PARAMS-1],*entry1[MAX_PARAMS],*p2entry[MAX_PARAMS-1][3],*label1[MAX_PARAMS],*label2[MAX_PARAMS-1],*colorbutton,*errlabel,*rlabel,*fentry;
static double integr_vals[MAX_EQUATIONS][2];

extern struct labels *labels_lang;
static GtkAccelGroup *menu_accel[3];

extern parser_context *pc;
extern double singlevals[MAX_EQUATIONS][MAX_PARAMS];

static void on_singleval_change(void);
void main_window(void);

/* New functions to make variables static */
void mainwin_accels_add(GtkWindow *w){
		gtk_window_add_accel_group(w,(GtkAccelGroup*)menu_accel[0]);
		gtk_window_add_accel_group(w,(GtkAccelGroup*)menu_accel[1]);
		gtk_window_add_accel_group(w,(GtkAccelGroup*)menu_accel[2]);
}

void fentry_read_pc(void){
	gtk_entry_set_text((GtkEntry*)fentry,pc[curr_eq].fn_string);
}

static gboolean funcbuttonclick(GtkButton *b,gpointer in){
	int pos=gtk_editable_get_position(GTK_EDITABLE(fentry));
	gtk_editable_delete_selection(GTK_EDITABLE(fentry));
	gtk_editable_insert_text(GTK_EDITABLE(fentry),(char*)in,strlen((char*)in),&pos);
	gtk_editable_set_position(GTK_EDITABLE(fentry),pos);
}

/* Funcbutton - integrated in main window */
GtkWidget* funcbuttonfield(void){
	int i;
	static GtkWidget *win,*table,*button[38];
	static const char items[38][7]={
		"sin","cos","tan","sinh","cosh","tanh",
		"arcsin","arccos","arctan",
		"sqrt","ln","lg","exp",
		"round","floor","ceil","abs",
		"min","max",";",
		"1","2","3","4","5","6","7","8","9","0",
		"+","-","*","/","%","^","(",")"
	};
	static int instances;
	if(instances>0){
		return win;
	}
	win=gtk_frame_new("");
	gtk_container_set_border_width(GTK_CONTAINER(win),10);
	table=gtk_table_new(6,6,TRUE);
	gtk_container_add(GTK_CONTAINER(win),table);

	for (i=0;i<38;++i){
		button[i]=gtk_button_new_with_label(items[i]);
		g_signal_connect(G_OBJECT(button[i]),"clicked",G_CALLBACK(funcbuttonclick),(void*)items[i]);
		gtk_table_attach_defaults(GTK_TABLE(table),button[i],i%7,i%7+1,i/7,i/7+1);
	}
	g_signal_connect(G_OBJECT(win),"delete-event",G_CALLBACK(gtk_widget_hide_on_delete),NULL);
	++instances;
	return win;
	
}

/*Main window functions/callbacks*/
static gboolean on_drawrange_change(GtkWidget *w,GtkWidget *e[2]){
	int i;
	bool error=false;
	entry_check_valvar(e,2,&error);
	if(!error){
		pc[curr_eq].drawbordersx[0]=atof(gtk_entry_get_text((GtkEntry*)e[0]));
		pc[curr_eq].drawbordersx[1]=atof(gtk_entry_get_text((GtkEntry*)e[1]));
		if(pc[curr_eq].drawbordersx[0]>pc[curr_eq].drawbordersx[1]){
			double tmp=pc[curr_eq].drawbordersx[0];
			pc[curr_eq].drawbordersx[0]=pc[curr_eq].drawbordersx[1];
			pc[curr_eq].drawbordersx[1]=tmp;
		}
	}
	return TRUE;
}

static gboolean dborders_eq(GtkWidget *w,GtkEntry *e[2]){
	char x[40];
	snprintf(x,40,"%f",pc[curr_eq].drawbordersx[0]);
	gtk_entry_set_text(e[0],x);
	snprintf(x,40,"%f",pc[curr_eq].drawbordersx[1]);
	gtk_entry_set_text(e[1],x);
	redraw_graph();
	return TRUE;
}

static void param_entries(void){
	int i;
	char errtxt[40], tmp[2]="";
	const char *fnstr;
	parser_context *lpc=pc+curr_eq;
	
	free(lpc->par.values);
	lpc->par.values=NULL;
	lpc->par.num=0;
	subfn_free(lpc->ptrtree);
	free(lpc->ptrtree);

	fnstr=gtk_entry_get_text(GTK_ENTRY(fentry));
	gtk_label_set_text(GTK_LABEL(errlabel),"");

	free(lpc->fn_string);
	if(curr_eq!=MAX_EQUATIONS-1)
		lpc->fn_string=strdup(fnstr);
	else{/* prepend "f=" for calc mode to have a valid function*/
		lpc->fn_string=(char*)g_malloc(strlen(fnstr)+strlen("f=")+1);
		sprintf(lpc->fn_string,"f=%s",fnstr);
	}
	
	for(i=0;i<MAX_PARAMS;++i){
		gtk_widget_hide(entry1[i]);
		gtk_widget_hide(label1[i]);
		if(i>0)gtk_widget_hide(box[i-1]);
	}

	lpc->ptrtree=(subfunction*)calloc(1,sizeof(subfunction));
	if((i=einlesen(
	lpc->ptrtree,&lpc->par,lpc->fn_string)
	)!=0){
		snprintf(errtxt,40,labels_lang->parse_error,i);
		gtk_label_set_text(GTK_LABEL(errlabel),errtxt);
		free(lpc->ptrtree);lpc->ptrtree=NULL;
		/*if(GTK_IS_WIDGET(graph))*/redraw_graph();
		return;
	}
	
	for(i=0;i<MAX_PARAMS;++i)
	if(lpc->par.num>=i+1){
		tmp[0]=pc->par.names[i];
		gtk_label_set_text(GTK_LABEL(label1[i]),tmp);
		gtk_widget_show(entry1[i]);
		gtk_widget_show(label1[i]);
	}
	for(i=2;i<=lpc->par.num && i<=MAX_PARAMS;++i)
		gtk_widget_show(box[i-2]);
	if(pc->par.num>1)main_window();/*schar label change*/
	on_singleval_change();
	//on_schar_change();
	/*if(GTK_IS_WIDGET(graph))*/redraw_graph();
}

static void on_singleval_change(void){
	struct result res;
	int i;
	const char* tmp=NULL;
	GdkColor red={0,65535,0,0};

	if(!pc[curr_eq].ptrtree) return;
	
	for(i=0;i<MAX_PARAMS && i<pc[curr_eq].par.num;++i){
		tmp=gtk_entry_get_text(GTK_ENTRY(entry1[i]));
		char *endptr=NULL;
		(void)strtod(tmp,&endptr);
		if (endptr && strlen(endptr)>0){
			gtk_widget_modify_bg(entry1[i],GTK_STATE_NORMAL,&red);
		}
		else{
			gtk_widget_modify_bg(entry1[i],GTK_STATE_NORMAL,NULL);
			singlevals[curr_eq][i]=pc[curr_eq].par.values[i]=exec_simplestring(tmp,0);		
		}
	}

	res=auswerten(pc[curr_eq].ptrtree,&pc[curr_eq].par);
	tmp=(res.defined ? dbltocstr(res.value,20) : strdup(labels_lang->res_undef));
	gtk_label_set_text(GTK_LABEL(rlabel),tmp);
	if(pc[curr_eq].par.num>1)valuetable_set();
	free(tmp);
}
static gboolean on_schar_change(void){
	int i,j;
	bool error=false;
	if(!pc->ptrtree)return TRUE;
	for (i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i)
		entry_check_valvar(p2entry[i],3,&error);
	if(!error){
		for(i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i)
			if(pc[curr_eq].par.num>=i+2){
				pc[curr_eq].par_t[i].is=true;
				pc[curr_eq].par_t[i].begin=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][0])));
				pc[curr_eq].par_t[i].end=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][1])));
				pc[curr_eq].par_t[i].intvl=atof(gtk_entry_get_text(GTK_ENTRY(p2entry[i][2])));
			}
			else pc->par_t[i].is=false;

	}
	/*if(GTK_IS_WIDGET(graph))*/redraw_graph();
	return TRUE;
}


int aboutbox(void){
	GtkWidget *about,*txt;
	about=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	txt=gtk_label_new("Andrea's Funktionsrechner 0.5beta4\n (C) Andrea Oster, 2007,2008\n"
	#ifdef __WIN32__
	"Windows version - hardly tested - better use Linux ;)\n"
	#endif
	#ifdef __GNUC__
	"Compiled with GCC, version " __VERSION__ " on " __DATE__
	#endif
	);
	gtk_container_add(GTK_CONTAINER(about),txt);
	gtk_widget_show_all(about);
}

/*General stuff for changing of current equation: */
static GtkWidget *lstyle, *lstyle_len;
gboolean change_eq_nr(GtkComboBox *widget, GtkWidget *in){
	int i;
	char *tmp;
	if(widget)curr_eq=gtk_combo_box_get_active(widget);
	if(curr_eq>MAX_EQUATIONS) curr_eq=0;
	gtk_entry_set_text(GTK_ENTRY(fentry),(pc[curr_eq].fn_string ? pc[curr_eq].fn_string+(curr_eq==MAX_EQUATIONS-1 ? 2 : 0) : ""));
	for(i=0;i<pc[curr_eq].par.num && i<MAX_PARAMS;++i){
		tmp=dbltocstr(singlevals[curr_eq][i],100);
		gtk_entry_set_text(GTK_ENTRY(entry1[i]),tmp);
		free(tmp);
	}
	//specifics for calc mode
	if (curr_eq==MAX_EQUATIONS-1){
		if (in) gtk_widget_hide(in); /*dont show graph or single calc stuff */
	}
	else{
		if(in) gtk_widget_show(in);
		gtk_color_button_set_color(GTK_COLOR_BUTTON(colorbutton),&pc[curr_eq].col);
		redraw_graph();
	}
	on_singleval_change();
	if(curr_eq!=MAX_EQUATIONS-1){
		valuetable_set();
		gtk_toggle_button_set_active((GtkToggleButton*)lstyle,pc[curr_eq].linestyle);
		gtk_spin_button_set_value((GtkSpinButton*)lstyle_len,pc[curr_eq].dashlen);
		for (i=0;i<pc[curr_eq].par.num-1;++i)
		if(pc[curr_eq].par_t[i].is){
			char temp[20];
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].begin);
			gtk_entry_set_text((GtkEntry*)p2entry[i][0],temp);
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].end);
			gtk_entry_set_text((GtkEntry*)p2entry[i][1],temp);
			snprintf(temp,20,"%.9f",pc[curr_eq].par_t[i].intvl);
			gtk_entry_set_text((GtkEntry*)p2entry[i][2],temp);
		}
	}
	return FALSE;
}
/*
Needed by main window:

change_eq_nr() (ok)
on_drawrange_change() (needs entry_check_valvar() !!!)
dborders_eq() (ok)
on_schar_change() (needs entry_check_valvar() !!!)
menu_funcptr[][]-Funktionen:
new_wrapper(needs reset_all())
open_wrapper(needs openini())
save_wrapper(needs saveini())
vtab_export_wrapper,graph_export_wrapper(inkludiert)
print_wrapper,exit_wrapper
graph_points,graph_verticals,vector_stuff,graph_areas,graph_circles (ok, sind eh in _misc.h)
shell_window(inkludiert)
valuetable_init(inkludiert)
show_gwindow_full(inkludiert)
optiondialog(inkludiert)
help_wrapper,

aboutbox (ok)


pc
p2entry(hierher verschieben)
curr_eq(hierher verschieben)

Funktion, die feststellt, ob Graph zum Neuzeichnen da

*/
static gboolean lstyle_change(GtkToggleButton *c,GtkSpinButton *len){
	pc[curr_eq].linestyle=gtk_toggle_button_get_active(c) ? dashed : line;
	gtk_spin_button_set_value(len,pc[curr_eq].dashlen);
	redraw_graph();
	return TRUE;
}
static gboolean lstyle_len_change(GtkSpinButton *sb){
	pc[curr_eq].dashlen=gtk_spin_button_get_value_as_int(sb);
	return TRUE;
}
static gboolean cbutton_chcol(GtkWidget *w){
	gtk_color_button_get_color(GTK_COLOR_BUTTON(w),&pc[curr_eq].col);
	redraw_graph();
	return TRUE;
}
static gboolean calcintvl_change(GtkSpinButton *sb){
	pc[curr_eq].calcintvl=gtk_spin_button_get_value(sb);
	redraw_graph();
	return TRUE;
}

static gboolean ival(GtkSpinButton *b,int num){
	integr_vals[curr_eq][num-1]=gtk_spin_button_get_value(b);
	return FALSE;
}
static gboolean iqad1(GtkSpinButton *w,GtkLabel *r){
	double de,a;
	char *x;
	de=MIN(integr_vals[curr_eq][0],integr_vals[curr_eq][1]);
	a=MAX(integr_vals[curr_eq][0],integr_vals[curr_eq][1]);
	if(pc[curr_eq].ptrtree)
		x=dbltocstr(integrate_qad(pc[curr_eq].ptrtree,&pc[curr_eq].par,de,a)*(de<=a ? 1 : -1),12);
	gtk_label_set_text(r,pc[curr_eq].ptrtree ? x  : labels_lang->res_undef);
	return TRUE;
}
static gboolean wskal_change(GtkSpinButton *sb){
	pc[curr_eq].linewidth=gtk_spin_button_get_value_as_int(sb);
	redraw_graph();
	return FALSE;
}
void main_window(void){
	static GtkWidget *window,*button[2], *flabel,*table[4],*vpan,*ntab,*ntab_label[3],*widthskal,*widthlab,*widthbox,*drawbordersb,*drawbordersl[2],*drawborderse[2],
		*menualign,*menu, *menu_main_itemlist[4],*menu_file_items[4][7],*menu_main[4],*bexp,*butt;
	static struct{GtkWidget *box,*lab1,*intvl,*lab2;}calcintvl;
	static bool exists;
	int i,j;
	char tmp[50];
	static gboolean (*menu_funcptr[4][8])(void)={
		{new_wrapper,open_wrapper,save_wrapper,vtab_export_wrapper,graph_export_wrapper,print_wrapper,exit_wrapper},
		{graph_points,graph_verticals,vector_stuff,graph_areas,graph_circles,shell_window},
		{valuetable_init,show_gwindow_full,optiondialog,NULL,NULL,NULL,NULL,NULL},
		{help_wrapper,aboutbox,NULL,NULL,NULL,NULL,NULL,NULL},
	};
	static int menu_item_no[]={8,7,4,3};/*incl. captions!! */

	if(exists){
		for(i=0;i<3;++i)
			gtk_label_set_text(GTK_LABEL(ntab_label[i]),labels_lang->cap_tabs[i]);
		for(i=0;i<4;++i){
			gtk_label_set_text_with_mnemonic(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menu_main[i]))),labels_lang->menu_labels[i][0]);
			for(j=1;j<menu_item_no[i];++j){
				gtk_label_set_text_with_mnemonic(GTK_LABEL(gtk_bin_get_child(GTK_BIN(menu_file_items[i][j-1]))),labels_lang->menu_labels[i][j]);
				if(!isspace(labels_lang->menu_accels[i][j-1])){
					snprintf(tmp,50,"<AFRMain>/Menu%d/%d",i,j);
					gtk_accel_map_change_entry(tmp,labels_lang->menu_accels[i][j-1],GDK_CONTROL_MASK,TRUE);
				}
			}
		}
		for(i=1;i<=MAX_EQUATIONS-1;++i){
			gtk_combo_box_remove_text(GTK_COMBO_BOX(flabel),i-1);
			snprintf(tmp,15,"%s %d:",labels_lang->flabel_norm,i);
			gtk_combo_box_insert_text(GTK_COMBO_BOX(flabel),i-1,tmp);
		}
		gtk_combo_box_remove_text(GTK_COMBO_BOX(flabel),i-1);
		gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),labels_lang->flabel_calculator);
		gtk_combo_box_set_active(GTK_COMBO_BOX(flabel),curr_eq);
		
		gtk_expander_set_label((GtkExpander*)bexp,labels_lang->cap_funcbutton);
		
		gtk_label_set_text((GtkLabel*)drawbordersl[0],labels_lang->drawbordersx[0]);
		gtk_label_set_text((GtkLabel*)drawbordersl[1],labels_lang->drawbordersx[1]);
		
		gtk_label_set_text((GtkLabel*)calcintvl.lab1,labels_lang->calcintvl[0]);
		gtk_label_set_text((GtkLabel*)calcintvl.lab2,labels_lang->calcintvl[1]);

		for(i=0;i<pc[curr_eq].par.num-1 && i<MAX_PARAMS-1;++i){
			char tmp[strlen(labels_lang->schar_entry)];
			sprintf(tmp,labels_lang->schar_entry,pc[curr_eq].par.names[i+1]);
			gtk_label_set_text((GtkLabel*)label2[i],tmp);
		}
		gtk_spin_button_set_value((GtkSpinButton*)widthskal,pc[curr_eq].linewidth);
		return;
	}

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 10);
	gtk_window_set_title((GtkWindow*)window, "Andrea's Funktionsrechner 0.5beta4 (GTK2)");
	
	vpan=gtk_vpaned_new();
	for(i=0;i<4;++i){
		table[i]=gtk_table_new(7,3,FALSE);
		gtk_table_set_row_spacings(GTK_TABLE(table[i]),5);
		gtk_table_set_col_spacings(GTK_TABLE(table[i]),5);
	}
	ntab=gtk_notebook_new();
	for (i=0;i<3;++i){
		ntab_label[i]=gtk_label_new(labels_lang->cap_tabs[i]);
		gtk_notebook_append_page(GTK_NOTEBOOK(ntab),table[i+1],ntab_label[i]);
	}
	gtk_paned_pack1(GTK_PANED(vpan),table[0],FALSE,FALSE);
	gtk_paned_pack2(GTK_PANED(vpan),ntab,FALSE,FALSE);
	gtk_widget_show(ntab);
	
	gtk_container_add(GTK_CONTAINER(window), vpan);
	
	/*begin menu*/
	menualign=gtk_alignment_new(0,0,0,0);
	gtk_table_attach_defaults(GTK_TABLE(table[0]),menualign,0,3,0,1);

	menu=gtk_menu_bar_new();
	gtk_container_add(GTK_CONTAINER(menualign),menu);

	for(i=0;i<4;++i){
		menu_main_itemlist[i]=gtk_menu_new();
		menu_accel[i]=gtk_accel_group_new();
		for (j=1;j<menu_item_no[i];++j){
			snprintf(tmp,50,"<AFRMain>/Menu%d/%d",i,j);
			gtk_menu_set_accel_group((GtkMenu*)menu_main_itemlist[i],menu_accel[i]);
			menu_file_items[i][j-1]=gtk_image_menu_item_new_with_mnemonic(labels_lang->menu_labels[i][j]);
			gtk_widget_set_accel_path(menu_file_items[i][j-1],tmp,menu_accel[i]);
			if(!isspace(labels_lang->menu_accels[i][j-1]))
				gtk_accel_map_add_entry(tmp,labels_lang->menu_accels[i][j-1],GDK_CONTROL_MASK);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu_main_itemlist[i]),menu_file_items[i][j-1]);
			if(menu_funcptr[i][j-1])
				g_signal_connect(G_OBJECT(menu_file_items[i][j-1]),"activate",G_CALLBACK(menu_funcptr[i][j-1]),NULL);
		}

		menu_main[i]=gtk_menu_item_new_with_mnemonic(labels_lang->menu_labels[i][0]);
		gtk_menu_set_accel_group((GtkMenu*)menu_main_itemlist[i],menu_accel[i]);
		gtk_window_add_accel_group((GtkWindow*)window,menu_accel[i]);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_main[i]);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_main[i]),menu_main_itemlist[i]);
	}

	/*ugly but type-saving macro*/
#define menu_stock_image(I,J,STOCK)\
	gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_file_items[I][J],gtk_image_new_from_stock(STOCK,GTK_ICON_SIZE_MENU))
	
	menu_stock_image(0,0,GTK_STOCK_NEW);
	menu_stock_image(0,1,GTK_STOCK_OPEN);
	menu_stock_image(0,2,GTK_STOCK_SAVE_AS);
	menu_stock_image(0,5,GTK_STOCK_PRINT);
	menu_stock_image(0,6,GTK_STOCK_QUIT);
	menu_stock_image(2,2,GTK_STOCK_PREFERENCES);
#undef menu_stock_image
	/*end menu*/
	
	errlabel=gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table[0]), errlabel, 0,3, 2,3);
	
	flabel=gtk_combo_box_new_text();
	for (i=1;i<=MAX_EQUATIONS-1;++i){
		snprintf(tmp,15,"%s %d:",labels_lang->flabel_norm,i);
		gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),tmp);
	}
	gtk_combo_box_append_text(GTK_COMBO_BOX(flabel),labels_lang->flabel_calculator);
	gtk_combo_box_set_active(GTK_COMBO_BOX(flabel),curr_eq);
	g_signal_connect(G_OBJECT(flabel),"changed",G_CALLBACK(change_eq_nr),(void*)table[1]);
	gtk_table_attach_defaults(GTK_TABLE(table[0]),flabel,0,1, 4,5);
	
	fentry=gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(fentry),"f(x)=x");
	gtk_table_attach_defaults(GTK_TABLE(table[0]),fentry,2,3, 4,5);
	g_signal_connect(G_OBJECT(fentry),"changed",G_CALLBACK(param_entries),NULL);

	rlabel=gtk_label_new("");
	gtk_table_attach_defaults(GTK_TABLE(table[2]), rlabel,0,3, 0,1);

	for(i=0;i<MAX_PARAMS;++i){
		entry1[i]=gtk_entry_new();
		gtk_table_attach_defaults(GTK_TABLE(table[2]),entry1[i],2,3,2+2*i,3+2*i);
		gtk_entry_set_text(GTK_ENTRY(entry1[i]),"0");
		g_signal_connect(G_OBJECT(entry1[i]),"changed",G_CALLBACK(on_singleval_change),NULL);
		gtk_widget_set_no_show_all(entry1[i],TRUE);
		label1[i]=gtk_label_new("");
		gtk_table_attach_defaults(GTK_TABLE(table[2]),label1[i],0,1,2+2*i,3+2*i);
	}

	butt=funcbuttonfield();
	bexp=gtk_expander_new(labels_lang->cap_funcbutton);
	gtk_container_add((GtkContainer*)bexp,butt);
	gtk_table_attach_defaults((GtkTable*)table[0],bexp,0,3, 6,7);
	
	
	/* tab "drawing": */
	colorbutton=gtk_color_button_new();
	gtk_color_button_set_title(GTK_COLOR_BUTTON(colorbutton),labels_lang->color);
	gtk_table_attach_defaults(GTK_TABLE(table[1]), colorbutton, 0,1 ,1,2);
	g_signal_connect(G_OBJECT(colorbutton),"notify::color",G_CALLBACK(cbutton_chcol),NULL);
	
	GtkWidget *ls=gtk_hbox_new(FALSE,3);
	lstyle=gtk_check_button_new_with_label("- - - -");
	lstyle_len=gtk_spin_button_new_with_range(1,50,1);
	gtk_spin_button_set_value((GtkSpinButton*)lstyle_len,10);
	g_signal_connect((GObject*)lstyle,"clicked",(GCallback)lstyle_change,(void*)lstyle_len);
	g_signal_connect((GObject*)lstyle,"toggled",(GCallback)opt_subopt_active,(void*)lstyle_len);
	g_signal_connect((GObject*)lstyle_len,"value-changed",(GCallback)lstyle_len_change,NULL);
	gtk_box_pack_start_defaults((GtkBox*)ls,lstyle);
	gtk_box_pack_start_defaults((GtkBox*)ls,lstyle_len);
	gtk_table_attach_defaults((GtkTable*)table[1],ls, 1,2, 1,2);

	widthbox=gtk_vbox_new(FALSE,5);
	widthlab=gtk_label_new_with_mnemonic(labels_lang->linewidth);
	widthskal=gtk_spin_button_new_with_range(1,50,1);
	g_signal_connect((GObject*)widthskal,"value-changed",(GCallback)wskal_change,NULL);
	gtk_label_set_mnemonic_widget((GtkLabel*)widthlab,widthskal);
	gtk_box_pack_start_defaults((GtkBox*)widthbox,widthlab);
	gtk_box_pack_start_defaults((GtkBox*)widthbox,widthskal);
	gtk_table_attach_defaults((GtkTable*)table[1],widthbox,2,3, 1,2);

	drawbordersb=gtk_hbox_new(FALSE,4);
	drawbordersl[0]=gtk_label_new(labels_lang->drawbordersx[0]);
	drawbordersl[1]=gtk_label_new(labels_lang->drawbordersx[1]);
	for (i=0;i<2;++i){
		drawborderse[i]=gtk_entry_new();
		g_signal_connect((GObject*)drawborderse[i],"changed",(GCallback)on_drawrange_change,(void*)drawborderse);
		gtk_box_pack_start_defaults((GtkBox*)drawbordersb,drawbordersl[i]);
		gtk_box_pack_start_defaults((GtkBox*)drawbordersb,drawborderse[i]);
	}
	gtk_entry_set_text((GtkEntry*)drawborderse[0],"-100000");
	gtk_entry_set_text((GtkEntry*)drawborderse[1],"100000");
	gtk_table_attach_defaults((GtkTable*)table[1],drawbordersb,0,3, 3,4);
	g_signal_connect((GObject*)flabel,"changed",(GCallback)dborders_eq,(void*)drawborderse);
	
	calcintvl.box=gtk_hbox_new(FALSE,3);
	calcintvl.lab1=gtk_label_new(labels_lang->calcintvl[0]);
	calcintvl.intvl=gtk_spin_button_new_with_range(0.000,1000,0.001);
	calcintvl.lab2=gtk_label_new(labels_lang->calcintvl[1]);
	g_signal_connect((GObject*)calcintvl.intvl,"value-changed",(GCallback)calcintvl_change,NULL);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.lab1);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.intvl);
	gtk_box_pack_start_defaults((GtkBox*)calcintvl.box,calcintvl.lab2);
	gtk_table_attach_defaults((GtkTable*)table[1],calcintvl.box,0,3, 4,5);
	

	for(i=0;i<MAX_PARAMS-1;++i){
		box[i]=gtk_hbox_new(FALSE,5);
		gtk_widget_set_no_show_all(box[i],TRUE);
		label2[i]=gtk_label_new(labels_lang->schar_entry);
		gtk_box_pack_start_defaults(GTK_BOX(box[i]),label2[i]);
		gtk_widget_show(label2[i]);
		for (j=0;j<3;++j){
			p2entry[i][j]=gtk_entry_new();
			gtk_box_pack_start_defaults(GTK_BOX(box[i]),p2entry[i][j]);
			g_signal_connect(G_OBJECT(p2entry[i][j]),"changed",G_CALLBACK(on_schar_change),NULL);
			gtk_widget_show(p2entry[i][j]);
		}
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][0]),"-1");
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][1]),"1");
		gtk_entry_set_text(GTK_ENTRY(p2entry[i][2]),"1");
		gtk_table_attach_defaults(GTK_TABLE(table[1]),box[i],0,3,6+i,7+i);
	}
	
	/*iqad(integration) stuff:*/
	GtkWidget *ent[2],*res=gtk_label_new(""),
		  *lab1=gtk_label_new("von"),
		  *lab2=gtk_label_new("bis");
	ent[0]=gtk_spin_button_new_with_range(-1000,1000,0.1);
	ent[1]=gtk_spin_button_new_with_range(-1000,1000,0.1);
	gtk_table_attach_defaults((GtkTable*)table[3],res,0,3,0,1);
	gtk_table_attach_defaults((GtkTable*)table[3],lab1,0,1,2,3);
	gtk_table_attach_defaults((GtkTable*)table[3],ent[0],2,3,2,3);
	gtk_table_attach_defaults((GtkTable*)table[3],lab2,0,1,4,5);
	gtk_table_attach_defaults((GtkTable*)table[3],ent[1],2,3,4,5);

	g_signal_connect((GObject*)ent[0],"value-changed",G_CALLBACK(ival),(void*)1);
	g_signal_connect((GObject*)ent[1],"value-changed",G_CALLBACK(ival),(void*)2);

	g_signal_connect((GObject*)ent[0],"value-changed",G_CALLBACK(iqad1),res);
	g_signal_connect((GObject*)ent[1],"value-changed",G_CALLBACK(iqad1),res);
	g_signal_connect((GObject*)fentry,"changed",G_CALLBACK(iqad1),res);
	
	
	g_signal_connect(G_OBJECT(window),"delete-event",G_CALLBACK(exit_wrapper),NULL);
	gtk_widget_show_all(window);
	exists=true;
}
