/*math_parser_gtk2_misc.c*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>

#include <assert.h>

#include "math_parser_compat.h"
#include "math_parser_gtk2.h"
#include "math_parser_gtk2_misc.h"
#include "math_parser_gtk2_draw.h"
#include "math_parser_gtk2_helperfns.h"
#include "math_parser_gtk2_lang.h"

#define p_num 3
#define v_num 3
#define a_num 3
#define c_num 3

static struct dp{
	bool is;
	const char *n;
	PangoFontDescription *font;
	double val[2];
	GdkColor col;
}
dpoints[p_num];

static struct vertical{
       bool is;
       double valx;
       GdkColor col;
       const char *n,*font;
}
vert[v_num];

static struct area{
       bool is;
       double bord_x[2];
       uint16_t bord_eqs[2];
       GdkColor col;
       const char *n,*font;
}
area[a_num];

static struct circle{
	bool is,filled;
	double x,y,rad;
	GdkColor col;
	const char *n,*font;
}
circle[c_num];

static const char empty[1]="";

extern struct{
	bool options,points,verticals,areas,circles;
}visible;
extern struct labels *labels_lang;


int read_point(const char in[],int i){
	short int n=0;
	if(i>=p_num) return 2;
	if(sscanf(in,"%lf;%lf; %d;%d;%d;%hn",
		&dpoints[i].val[0],&dpoints[i].val[1],&dpoints[i].col.red,&dpoints[i].col.green,&dpoints[i].col.blue,&n)<5){
		dpoints[i].is=false;
		return 1;
	}
	else{
		dpoints[i].is=true;
		dpoints[i].n=strdup(in+n);
		if(visible.points)graph_points();
		return 0;
	}
}
int read_vertical(const char in[],int i){
	if(i>=v_num) return 2;
	return !(vert[i].is=(sscanf(in,"%lf; %d;%d;%d",
		&vert[i].valx,&vert[i].col.red,&vert[i].col.green,&vert[i].col.blue)==4));
}
int read_area(const char in[],int i){
	short int n=0;
	if (i>=a_num) return 2;
	if (sscanf(in,"%lf;%lf; %d;%d; %d;%d;%d;%hn",
		area[i].bord_x[0],area[i].bord_x[1],area[i].bord_eqs[0],area[i].bord_eqs[1],
		area[i].col.red,area[i].col.green,area[i].col.blue,n)!=7){
		area[i].is=false;
		return 1;
	}
	area[i].is=true;
	area[i].n=(char*)calloc(1,strlen(in+n)+1);/*i know thats more space than really needed (space for .n and .font together) */
	area[i].font=(char*)calloc(1,strlen(in+n)+1);/*but its safe and doesnt waste that much memory */
	sscanf(in+n,"%s;%s",area[i].n,area[i].font);
	if(visible.areas) graph_areas();
	return 0;
}
int read_circle(const char in[], int i){
	if (i>=c_num) return 2;
	if (sscanf(in,"CIRCLE=%.3f;.3%f; %.3f; %d;%d;%d;%i;",
		circle[i].x,circle[i].y, circle[i].rad,circle[i].col.red,circle[i].col.green,circle[i].col.blue,circle[i].filled)!=7){
		circle[i].is=false;
		return 1;
	}
	circle[i].is=true;
	if(visible.circles)graph_circles();
	return 0;
}	

char* dump_misc(void){
	char *out=NULL;
	char tmp[200];
	int i;
	for(i=0;i<p_num;++i)
		if (dpoints[i].is){	
			snprintf(tmp,200,"POINT=%f;%f; %d;%d;%d;%s\n",
				dpoints[i].val[0],dpoints[i].val[1],dpoints[i].col.red,dpoints[i].col.green,dpoints[i].col.blue,dpoints[i].n ? dpoints[i].n : "");
			if (!(out=(char*)(out ? g_realloc(out,strlen(out)+strlen(tmp)+1) : g_malloc0(strlen(tmp)+1))));
			strcat(out,tmp);
		}
	
	for(i=0;i<v_num;++i)
		if(vert[i].is){
			snprintf(tmp,200,"VERT=%f; %d;%d;%d\n",
				vert[i].valx,vert[i].col.red,vert[i].col.green,vert[i].col.blue);
			out=(char*)(out ? g_realloc(out,strlen(out)+strlen(tmp)+1) : g_malloc0(strlen(tmp)+1));
			strcat(out,tmp);
		}

	for (i=0;i<a_num;++i)
		if(area[i].is){
			snprintf(tmp,200,"AREA=%f;%f; %d;%d; %d;%d;%d; %s;%s\n",
				area[i].bord_x[0],area[i].bord_x[1],area[i].bord_eqs[0],area[i].bord_eqs[1],
				area[i].col.red,area[i].col.green,area[i].col.blue,area[i].n, area[i].font);
			out=(char*)(out ? g_realloc(out,strlen(out)+strlen(tmp)+1) : g_malloc0(strlen(tmp)+1));
			strcat(out,tmp);
		}
	for (i=0;i<c_num;++i)
		if (circle[i].is){
			
			snprintf(tmp,200,"CIRCLE=%.3f;.3%f; %.3f; %d;%d;%d;%i\n",
				circle[i].x,circle[i].y, circle[i].rad,
      				circle[i].col.red,circle[i].col.green,circle[i].col.blue,circle[i].filled);
			out=(char*)(out ? g_realloc(out,strlen(out)+strlen(tmp)+1) : g_malloc0(strlen(tmp)+1));
			strcat(out,tmp);
		}
	return out ? out : (char*) empty;
}

static gboolean dpoints_change_font(GtkFontButton *fb,unsigned int idx){
	assert(idx<p_num);
	if(dpoints[idx].font) pango_font_description_free(dpoints[idx].font);
	dpoints[idx].font=pango_font_description_from_string(gtk_font_button_get_font_name(fb));
}
static gboolean dpoints_change_val(GtkEntry *null,GtkEntry *ent[3][3]){
	 int i,j;
	 for (i=0;i<p_num;++i){
	 	free(dpoints[i].n);
		dpoints[i].n=strdup(gtk_entry_get_text(ent[i][0]));
	 	for(j=1;j<3;++j){
			char *endptr=NULL,*entval=gtk_entry_get_text(ent[i][j]);
			if(!entval || !(*entval)){
				dpoints[i].is=false;break;
			}
			dpoints[i].val[j-1]=strtod(entval,&endptr);
			if ((endptr && *endptr) || !isfinite(dpoints[i].val[j-1])){
				dpoints[i].is=false;break;
			}
			else dpoints[i].is=true;
		}
	 }
	 redraw_graph();
	 return TRUE;
}
void graph_points(void){
	static GtkWidget *win,*tab,*ent[3][3],*lab[2],*cbut[p_num],*fbut[p_num];
	static int instances;
	size_t i,j;
	if(instances){
		gtk_widget_hide(win);
		gtk_widget_show_all(win);
		gtk_window_set_title((GtkWindow*)win,labels_lang->points[0]);
		gtk_label_set_text((GtkLabel*)lab[0],labels_lang->points[1]);
		gtk_label_set_text((GtkLabel*)lab[1],labels_lang->points[2]);

		for(i=0;i<p_num;++i)
			if(dpoints[i].is){
				char tmp[20];
				gtk_entry_set_text((GtkEntry*)ent[i][0],dpoints[i].n);
				for(j=1;j<3;++j){
					snprintf(tmp,20,"%f",dpoints[i].val[j-1]);
					gtk_entry_set_text((GtkEntry*)ent[i][j],tmp);
				}
				gtk_color_button_set_color((GtkColorButton*)cbut[i],&dpoints[i].col);
			}
		return;
	}
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title((GtkWindow*)win,labels_lang->points[0]);
	tab=gtk_table_new(4,9,FALSE);
	gtk_container_add((GtkContainer*)win,tab);

	lab[0]=gtk_label_new(labels_lang->points[1]);
	lab[1]=gtk_label_new(labels_lang->points[2]);
	gtk_table_attach_defaults((GtkTable*)tab,lab[0],0,1,0,1);
	gtk_table_attach_defaults((GtkTable*)tab,lab[1],2,5,0,1);

	for (i=0;i<p_num;++i){
		for (j=0;j<3;++j){
			ent[i][j]=gtk_entry_new();
			g_signal_connect((GObject*)ent[i][j],"changed",(GCallback)dpoints_change_val,(void*)ent);
			gtk_table_attach_defaults((GtkTable*)tab,ent[i][j],j*2,j*2+1,i+1,i+2);
		}
		GdkColor black={0,0,0};
		cbut[i]=gtk_color_button_new_with_color(&black);
		g_signal_connect((GObject*)cbut[i],"color-set",(GCallback)colorbutton_changed,(void*)&dpoints[i].col);
		gtk_table_attach_defaults((GtkTable*)tab,cbut[i],6,7,i+1,i+2);
		
		fbut[i]=gtk_font_button_new();
		g_signal_connect((GObject*)fbut[i],"font-set",(GCallback)dpoints_change_font,(void*)i);
		gtk_table_attach_defaults((GtkTable*)tab,fbut[i],7,9,i+1,i+2);
	}
	g_signal_connect(G_OBJECT(win),"delete_event",(GCallback)bool_set_false,(void*)&visible.points);
	visible.points=true;
	
	g_signal_connect(G_OBJECT(win),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),NULL);
	//gtk_widget_set_size_request((GtkWidget*)win,250,100);
	++instances;
	graph_points();
}
gboolean draw_gdk_points(graph_cache *gcache,int skalx,int skaly,bool blackwhite){
	int i;
	GdkGC *gc=gdk_gc_new(gcache->pm);
	PangoLayout *pangolayout=pango_layout_copy(gcache->pango);
	struct {int x,y;}ursprung={zero_x(),zero_y()};
	for (i=0;i<p_num;++i)
		if(dpoints[i].is){
			if(!blackwhite)gdk_gc_set_rgb_fg_color(gc,&dpoints[i].col);
			gdk_gc_set_line_attributes(gc,2,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
			gdk_draw_line(gcache->pm,gc,x_gdk(dpoints[i].val[0])-3,y_gdk(dpoints[i].val[1])-3,x_gdk(dpoints[i].val[0])+3,y_gdk(dpoints[i].val[1])+3);
			gdk_draw_line(gcache->pm,gc,x_gdk(dpoints[i].val[0])+3,y_gdk(dpoints[i].val[1])-3,x_gdk(dpoints[i].val[0])-3,y_gdk(dpoints[i].val[1])+3);

			if(dpoints[i].font) pango_layout_set_font_description(pangolayout,dpoints[i].font);
			pango_layout_set_text(pangolayout,dpoints[i].n,-1);
			gdk_draw_layout(gcache->pm,gc,x_gdk(dpoints[i].val[0])-5,y_gdk(dpoints[i].val[1])+5,pangolayout);
		}
	g_object_unref(pangolayout);
}
#ifdef USE_GTK2_PRINT
void draw_cairo_points(cairo_t *cr,float skalx,float skaly,struct root ursprung,bool blackwhite){
	int i;
	for (i=0;i<p_num;++i)
		if(dpoints[i].is){
			if(!blackwhite)cairo_set_source_rgb(cr,dpoints[i].col.red/65535.0,dpoints[i].col.green/65535.0,dpoints[i].col.blue/65535.0);
			cairo_move_to(cr,x_cairo(dpoints[i].val[0])-2,y_cairo(dpoints[i].val[1])-2);
			cairo_line_to(cr,x_cairo(dpoints[i].val[0])+2,y_cairo(dpoints[i].val[1])+2);
			cairo_move_to(cr,x_cairo(dpoints[i].val[0])+2,y_cairo(dpoints[i].val[1])-2);
			cairo_line_to(cr,x_cairo(dpoints[i].val[0])-2,y_cairo(dpoints[i].val[1])+2);
		}
	cairo_stroke(cr);
}
#endif

static gboolean graph_verticals_change1(GtkEntry *e,size_t i){
	bool err=false;
	entry_check_valvar(&e,1,&err);
	if(!err){
		vert[i].is=true;
		vert[i].valx=atof(gtk_entry_get_text(e));
	}
	else	vert[i].is=false;
	redraw_graph();
}
void graph_verticals(void){
	static GtkWidget *win,*tab,*lab[v_num],*val[v_num],*col[v_num];
	static int instances;
	size_t i;

	if(instances){
		gtk_widget_hide(win);
		gtk_widget_show_all(win);
		gtk_window_set_title((GtkWindow*)win,labels_lang->vertical[0]);
		for(i=0;i<v_num;++i){
			char tmp[20];
			gtk_label_set_text((GtkLabel*)lab[i],labels_lang->vertical[1]);
			if(vert[i].is){
				snprintf(tmp,20,"%f",vert[i].valx);
				gtk_entry_set_text((GtkEntry*)val[i],tmp);
				gtk_color_button_set_color((GtkColorButton*)col[i],&vert[i].col);
			}
		}
		return;
	}
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title((GtkWindow*)win,labels_lang->vertical[0]);
	tab=gtk_table_new(3,3,TRUE);
	gtk_container_add((GtkContainer*)win,tab);
	for (i=0;i<v_num;++i){
		lab[i]=gtk_label_new(labels_lang->vertical[1]);
		val[i]=gtk_entry_new();
		col[i]=gtk_color_button_new();
		gtk_table_attach_defaults((GtkTable*)tab,lab[i],0,1,i,i+1);
		gtk_table_attach_defaults((GtkTable*)tab,val[i],1,2,i,i+1);
		gtk_table_attach_defaults((GtkTable*)tab,col[i],3,4,i,i+1);
		g_signal_connect((GObject*)val[i],"changed",(GCallback)graph_verticals_change1,(void*)i);
		g_signal_connect((GObject*)col[i],"color-set",(GCallback)colorbutton_changed,(void*)&vert[i].col);
	}
	g_signal_connect((GObject*)win,"delete_event",(GCallback)bool_set_false,(void*)&visible.verticals);
	visible.verticals=true;
	
	g_signal_connect(G_OBJECT(win),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),NULL);
	++instances;
	graph_verticals();
}
gboolean draw_gdk_verticals(graph_cache *gcache,int skalx,int skaly,bool blackwhite){
	int i;
	GdkGC *gc=gdk_gc_new(gcache->pm);
	struct {int x,y;} ursprung={zero_x(),zero_y()};
	gdk_gc_set_line_attributes(gc,2,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
	for (i=0;i<v_num;++i)
		if(vert[i].is){
			if(!blackwhite)gdk_gc_set_rgb_fg_color(gc,&vert[i].col);
			gdk_draw_line(gcache->pm,gc,x_gdk(vert[i].valx),0,x_gdk(vert[i].valx),10000);
		}
	g_object_unref(gc);
}
#ifdef USE_GTK2_PRINT
void draw_cairo_verticals(cairo_t *cr,float skalx,float skaly,struct root ursprung,bool blackwhite){
	int i,x=cairo_get_line_width(cr);
	cairo_set_line_width(cr,x*2);
	for (i=0;i<v_num;++i)
		if(vert[i].is){
			if(!blackwhite)cairo_set_source_rgb(cr,vert[i].col.red/65535.0,vert[i].col.green/65535.0,vert[i].col.blue/65535.0);
			cairo_move_to(cr,vert[i].valx,0);
			cairo_line_to(cr,vert[i].valx,HUGE_VAL);
		}
	cairo_stroke(cr);
	cairo_set_line_width(cr,x);
}
#endif


static void graph_areas_change_num(GtkEntry *n,GtkEntry *e[a_num][2]){
	int i,j;
	char *endptr=NULL;
	for(i=0;i<a_num;++i){
		area[i].bord_x[0]=strtod(gtk_entry_get_text(e[i][0]),&endptr);
		if(endptr && *endptr) area[i].is=false;
		area[i].bord_x[1]=strtod(gtk_entry_get_text(e[i][1]),&endptr);
		if(endptr && *endptr) area[i].is=false;
	}
	redraw_graph();
}
static void graph_areas_change_eqs(GtkSpinButton *n,GtkSpinButton *sb[a_num][2]){
	int i,j;
	for (i=0;i<a_num;++i){
		area[i].bord_eqs[0]=gtk_spin_button_get_value_as_int(sb[i][0]);
		area[i].bord_eqs[1]=gtk_spin_button_get_value_as_int(sb[i][1]);
		if(area[i].bord_eqs[0]==area[i].bord_eqs[1]) area[i].is=false;
	}
	redraw_graph();
}
void graph_areas(void){
	static GtkWidget *win,*tab,*lab[2],*eq_choice[a_num][2],*num_choice[a_num][2],*col[a_num];
	static int instances;
	int i,j;
	if (instances){
		gtk_widget_hide(win);
		gtk_widget_show_all(win);
		gtk_window_set_title((GtkWindow*)win,labels_lang->area[0]);
		return;
	}
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title((GtkWindow*)win,labels_lang->area[0]);
	tab=gtk_table_new(5,1+a_num,TRUE);
	gtk_container_add((GtkContainer*)win,tab);
	for(i=0;i<2;++i){	
		lab[i]=gtk_label_new(labels_lang->area[i+1]);
		gtk_table_attach_defaults((GtkTable*)tab,lab[i],i*2,i*2+1,0,1);
	}
	for(i=0;i<a_num;++i){
		col[i]=gtk_color_button_new();
		for(j=0;j<2;++j){
			num_choice[i][j]=gtk_entry_new();
			eq_choice[i][j]=gtk_spin_button_new_with_range(0,MAX_EQUATIONS,1);
			g_signal_connect((GObject*)eq_choice[i][j],"value-changed",(GCallback)graph_areas_change_eqs,(void*)eq_choice);
			g_signal_connect((GObject*)num_choice[i][j],"changed",(GCallback)graph_areas_change_num,(void*)num_choice);
			gtk_table_attach_defaults((GtkTable*)tab,num_choice[i][j],j+2,j+3,i+1,i+2);
			gtk_table_attach_defaults((GtkTable*)tab,eq_choice[i][j],j,j+1,i+1,i+2);
		}
		g_signal_connect((GObject*)col[i],"color-set",(GCallback)colorbutton_changed,(void*)&area[i].col);
		gtk_table_attach_defaults((GtkTable*)tab,col[i],4,5,1+i,2+i);
	}
	g_signal_connect((GObject*)win,"delete_event",(GCallback)bool_set_false,(void*)&visible.areas);
	visible.areas=true;
	
	g_signal_connect(G_OBJECT(win),"delete_event",G_CALLBACK(gtk_widget_hide_on_delete),NULL);
	gtk_widget_show_all(win);
	++instances;
}
void draw_gdk_areas(graph_cache *gcache,int skalx,int skaly,bool blackwhite,parser_context *pc){
	int i,d,j;
	struct result res[2];
	struct root ursprung={zero_x(),zero_y()};
	GdkGC *gc=gdk_gc_new(gcache->pm);
	for(i=0;i<a_num;++i){
		const uint16_t *eqnr=(const uint16_t*)&area[i].bord_eqs;
		if(eqnr[0]==eqnr[1]) continue;
		if(!blackwhite)gdk_gc_set_rgb_fg_color(gc,&area[i].col);
		for(d=x_gdk(area[i].bord_x[0]);d<=x_gdk(area[i].bord_x[1]);++d){
			for(j=0;j<2;++j){
				if(eqnr[j]==0) res[j].value=0;
				else if(pc[eqnr[j]-1].ptrtree){
					if(pc[eqnr[j]-1].par.num>0)pc[eqnr[j]-1].par.values[0]=x_gdk_reverse(d);
					res[j]=subfn_evaluate(pc[eqnr[j]-1].ptrtree,&pc[eqnr[j]-1].par);
				}
			}
			gdk_draw_line(gcache->pm,gc,d,y_gdk(res[0].value),d,y_gdk(res[1].value));
		}
	}
	g_object_unref(gc);
}
#ifdef USE_GTK2_PRINT
void draw_cairo_areas(cairo_t *cr,float skalx,float skaly,struct root ursprung,bool blackwhite,parser_context *pc){
	int i,d,j;
	struct result res[2];
	for(i=0;i<a_num;++i){
		const uint16_t *eqnr=(const uint16_t*)&area[i].bord_eqs;
		if(eqnr[0]==eqnr[1]) continue;
		if(!blackwhite)cairo_set_source_rgb(cr,area[i].col.red/65536.0,area[i].col.green/65536.0,area[i].col.blue/65536.0);
		for(d=x_gdk(area[i].bord_x[0]);d<=x_gdk(area[i].bord_x[1]);++d){
			for(j=0;j<2;++j){
				if(eqnr[j]==0) res[j].value=0;
				else if(pc[eqnr[j]-1].ptrtree){
					if(pc[eqnr[j]-1].par.num>0)pc[eqnr[j]-1].par.values[0]=x_gdk_reverse(d);
					res[j]=subfn_evaluate(pc[eqnr[j]-1].ptrtree,&pc[eqnr[j]-1].par);
				}
			}
			cairo_move_to(cr,d,y_gdk(res[0].value));
			cairo_line_to(cr,d,y_gdk(res[1].value));
		}
	}
}
#endif

/*Circles*/
static struct {GtkWidget *box, *active, *pos_x, *pos_y, *rad, *color, *filled;} c_ctrl[c_num];
void graph_circles_change(GtkWidget *w,int i){
	circle[i].is=gtk_toggle_button_get_active((GtkToggleButton*)c_ctrl[i].active);
	circle[i].filled=gtk_toggle_button_get_active((GtkToggleButton*)c_ctrl[i].filled);
	circle[i].x=gtk_spin_button_get_value((GtkSpinButton*)c_ctrl[i].pos_x);
	circle[i].y=gtk_spin_button_get_value((GtkSpinButton*)c_ctrl[i].pos_y);
	circle[i].rad=gtk_spin_button_get_value((GtkSpinButton*)c_ctrl[i].rad);
	
	redraw_graph();
}
void graph_circles(void){//TODO subopt
	static int i,instances;
	static GtkWidget *win,*wtab;
	if(instances){
		for(i=0;i<c_num;++i){
			gtk_toggle_button_set_active((GtkToggleButton*)c_ctrl[i].active,circle[i].is);
			gtk_toggle_button_set_active((GtkToggleButton*)c_ctrl[i].filled,circle[i].filled);
		}
	}
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	wtab = gtk_table_new(1,c_num,TRUE);
	gtk_container_add((GtkContainer*)win,wtab);
	for(i=0;i<c_num;++i){
		c_ctrl[i].box=gtk_hbox_new(FALSE,3);
		c_ctrl[i].active=gtk_check_button_new_with_mnemonic("Kreis zeichnen mit Radius");
		gtk_toggle_button_set_active((GtkToggleButton*)c_ctrl[i].active,FALSE);
		c_ctrl[i].pos_x=gtk_spin_button_new_with_range(-99999,99999,0.001);
		c_ctrl[i].pos_y=gtk_spin_button_new_with_range(-99999,99999,0.001);
		c_ctrl[i].rad=gtk_spin_button_new_with_range(0,99999,0.001);
		c_ctrl[i].color=gtk_color_button_new();
		c_ctrl[i].filled=gtk_check_button_new_with_label("Fuellen");
		
		g_signal_connect((GObject*)c_ctrl[i].active,"toggled",(GCallback)graph_circles_change,(void*)i);
		g_signal_connect((GObject*)c_ctrl[i].rad,"value-changed",(GCallback)graph_circles_change,(void*)i);
		g_signal_connect((GObject*)c_ctrl[i].pos_x,"value-changed",(GCallback)graph_circles_change,(void*)i);
		g_signal_connect((GObject*)c_ctrl[i].pos_y,"value-changed",(GCallback)graph_circles_change,(void*)i);
		g_signal_connect((GObject*)c_ctrl[i].color,"color-set",(GCallback)graph_circles_change,(void*)i);
		g_signal_connect((GObject*)c_ctrl[i].filled,"toggled",(GCallback)graph_circles_change,(void*)i);
		
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].active);
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].rad);
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].pos_x);
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].pos_y);
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].color);
		gtk_box_pack_start_defaults((GtkBox*)c_ctrl[i].box,c_ctrl[i].filled);

		gtk_table_attach_defaults((GtkTable*)wtab,c_ctrl[i].box,0,1,i,i+1);
	}
	gtk_widget_show_all(win);
	++instances;
}

void draw_gdk_circles(graph_cache *gcache,int skalx,int skaly,bool blackwhite){
	int i,x,y,w,h;	
	GdkGC *gc=gdk_gc_new(gcache->pm);
	struct root ursprung={zero_x(), zero_y()};
	for (i=0;i<c_num;++i){if(!circle[i].is) continue;
		x=y=w=h=300;
		gdk_draw_arc(gcache->pm,gc,circle[i].filled,x_gdk(circle[i].x)-circle[i].rad*skalx/2.0,y_gdk(circle[i].y)-circle[i].rad*skaly/2.0,circle[i].rad*skalx,circle[i].rad*skaly,0,360*64);
	}
}
#ifdef USE_GTK2_PRINT
void draw_cairo_circles(cairo_t *cr,float skalx,float skaly,struct root ursprung,bool blackwhite,parser_context *pc){
	for (int i=0;i<c_num;++i){
		if (!circle[i].is) continue;
		cairo_arc(cr,x_cairo(circle[i].x)-circle[i].rad*skalx/2.0,y_cairo(circle[i].y)-circle[i].rad*skaly/2.0,circle[i].rad*skalx,circle[i].rad*skaly,0,360*64);
	}
}
#endif

/* Folgen einbauen:

- z. B. x(n+1)= x(n)*r
- extra Fenster wo nur x / x*r eingeg. wird
- besser: als Funktion, die mit ; eingeleitet wird -> Zeichen, dass Formel nicht direkt an Parser geht

*/


/*Vector calculation add-on:*/
struct vector_change{
	GtkSpinButton *sb[2][3];
	GtkLabel *l[2];
};
gboolean vector_change(GtkSpinButton *n, struct vector_change* vc){
	float v[2][3];
	char tmp[50];
	int i,j;
	for(i=0;i<2;++i)
		for(j=0;j<3;++j)
			v[i][j]=gtk_spin_button_get_value(vc->sb[i][j]);
	/*Vektor(skalar)produkt:*/
	snprintf(tmp,50,"Skalarprodukt: %f",
		v[0][0]*v[1][0]+v[0][1]*v[1][1]+v[0][2]*v[1][2]);
	gtk_label_set_text(vc->l[0],tmp);
	/*Kreuzprodukt:*/
	snprintf(tmp,50,"Kreuzprodukt:\n"
			" %f\n"
			" %f\n"
			" %f\n",
		v[0][1]*v[1][2]-v[0][2]*v[1][1],
		v[0][2]*v[1][0]-v[0][0]*v[1][2],
		v[0][0]*v[1][1]-v[0][1]*v[1][0]);
	gtk_label_set_text(vc->l[1],tmp);

}
void vector_stuff(void){
	int i,j;
	static GtkWidget *win,*tab,*vec[2][3],*lab[2];
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	tab=gtk_table_new(4,3,TRUE);
	gtk_container_add((GtkContainer*)win,tab);
	static struct vector_change vc;
	for(i=0;i<2;++i){
		lab[i]=gtk_label_new("");
		gtk_table_attach_defaults((GtkTable*)tab,lab[i],0+i,1+i,5,6);
		vc.l[i]=(GtkLabel*)lab[i];
		for(j=0;j<3;++j){
			vec[i][j]=gtk_spin_button_new_with_range(-1000,1000,0.5);
			vc.sb[i][j]=(GtkSpinButton*)vec[i][j];
			g_signal_connect((GObject*)vec[i][j],"value-changed",(GCallback)vector_change,(void*)&vc);
			gtk_table_attach_defaults((GtkTable*)tab,vec[i][j],i,i+1,j,j+1);
		}
	}
	gtk_widget_show_all(win);
}
/*Stochastik hypothesis add-on:*/
static GtkWidget *hypothesis_result;
gboolean hypothesis_change(GtkWidget *entry,GtkWidget *entries[4]){
	float n,p,q,c,bounds[2];
	char tmp[50];
	n=atoi(gtk_entry_get_text((GtkEntry*)entries[0]));
	p=atof(gtk_entry_get_text((GtkEntry*)entries[1]));
	q=1.000-p;
	c=atof(gtk_entry_get_text((GtkEntry*)entries[2]));
	if (p<0 || p>1) {gtk_label_set_text((GtkLabel*)hypothesis_result,"Error: p is wrong");return FALSE;}
	bounds[0]=(n*p)-sqrt(n*p*q)*c;
	bounds[1]=(n*p)+sqrt(n*p*q)*c;
	snprintf(tmp,50,"(+)={%d,...,%d}",MAX((int)ceil(bounds[0]),0),(int)floor(bounds[1]));
	gtk_label_set_text((GtkLabel*)hypothesis_result,tmp);
}

void hypothesis_stuff(void){
	static char *l_caps[4]={"n =","p =","c =","distribution: normal/ca. binomial"};
	static int instances;
	static GtkWidget *win,*tab,*label[4],*entry[4];
	if(instances) {gtk_widget_show_all(win); return;}
	win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	tab=gtk_table_new(4,2,TRUE);
	gtk_container_add((GtkContainer*)win,tab);
	for (int i=0;i<4;++i){
		label[i]=gtk_label_new(l_caps[i]);
		entry[i]=gtk_entry_new();
		g_signal_connect((GObject*)entry[i],"changed",(GCallback)hypothesis_change,(void*)entry);
	}
	hypothesis_result=gtk_label_new("");
	gtk_table_attach_defaults((GtkTable*)tab,label[0],0,1,0,1);
	gtk_table_attach_defaults((GtkTable*)tab,entry[0],1,2,0,1);
	gtk_table_attach_defaults((GtkTable*)tab,label[1],0,1,1,2);
	gtk_table_attach_defaults((GtkTable*)tab,entry[1],1,2,1,2);
	gtk_table_attach_defaults((GtkTable*)tab,label[2],0,1,2,3);
	gtk_table_attach_defaults((GtkTable*)tab,entry[2],1,2,2,3);
	gtk_table_attach_defaults((GtkTable*)tab,label[3],0,1,3,4);
	gtk_table_attach_defaults((GtkTable*)tab,hypothesis_result,0,2,4,5);
	instances++;
	gtk_widget_show_all(win);
}
