/*math_parser_helperfns.c */
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <locale.h>

#include "math_parser.h"

/***************************************************************************
 *   Copyright (C) 2007,2008 by Andrea Oster				   *
 *   ostera@googlemail.com						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License, version 2, as   *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

extern const struct func fn;

bool isoperator(char in){
	switch(in){
		case '+': case '-':
		case '*': case '/':
		case '%': case '^': 	return true;
	}
	return false;
}


void strins(char* str,size_t pos,const char ins[]){
	str[strlen(str)+strlen(ins)]='\0';
	memmove(str+pos+strlen(ins),str+pos,strlen(str)-pos);
	memcpy(str+pos,ins,strlen(ins));
}

void strpartcpy(const char input[], const size_t begin, const size_t end, char* output){
	size_t i;
	for (i=begin;i<=end;++i)
		output[i-begin]=input[i];
}

uint32_t strdiv(const char in[], char* (* out[]),const char pattern[], const char block[]){
/* Divides inout _before_ the characters in pattern, ignoring these which are in between block chars.
 * Example: inout="3*x*(x+2*t)", pattern="*",block="()[]"
 * strdiv() output: "3", "*x","*(x+2*t)". The last '*' is not divided because it's in brackets.
 * If characters in pattern and block are the same, the block is checked first, therefore the other is ignored.
 * Note: out should be a not already allocated char**. free() it before using this function. Static char** 's are not usable for output.
 */ 	
	uint32_t i,j,outcount=0,begin=0,blockzaehler=0;
	if (!in || !pattern ||!block)return 0;
	if(out)	*out = (char**) calloc(1,sizeof(char*));
	for (i=0;i<strlen(in);++i){
		for (j=0;j<strlen(block);++j){
			if (in[i]==block[j]){
				if((j+1)%2==1) ++blockzaehler;
				else --blockzaehler;
				break;
			}
		}
		if (i+1==strlen(in)) ++i;
				/*We are dividing _before_ pattern characters, but at the end is no such character. 
				With ++i, we avoid omitting the last character; see the strpartcpy(...,i-1,..) call below */
		for (j=0;j<strlen(pattern);++j)
			if ((i!=0 && in[i]==pattern[j] && blockzaehler==0) || i==strlen(in)){
				if(out){/* if out is NULL, then strdiv() is useful for knowing if a split is appropriate */
					*out = (char**) realloc(*out, (outcount+1)*sizeof(char*));
					(*out)[outcount] = (char*) calloc(i-begin+1,sizeof(char));
					strpartcpy(in,begin,i-1,(*out)[outcount]);
				}
				++outcount;
				begin=i;
				break;
			}
	}
	return outcount;
}

uint32_t validate(const char* parsing,const struct params *p){
/* This function tests if the given null-terminated string  w/o spaces can be a valid term (right-side, w/o f(x) )
 * Return value: 0 if valid, -1/-2 for bracket count errors, positive values indicate the error place (counting starts w/ 1)
 */
	uint32_t i,j,lastfn=0;
	int32_t counter=0,
	namecounter=0;	/* counts current pos in fn name (e.g. sin) */
	bool numcolon=false;
	if (!parsing || strlen(parsing)==0)return -1;

	for (i=0;i<strlen(parsing);++i){
		if(counter==0 && namecounter==0) 
			lastfn=0;
		if(!isdigit(parsing[i]) && parsing[i]!=',' && parsing[i]!='.')
			numcolon=false;
		switch (parsing[i]){
			case '.': /* expressions like ".3" are allowed */ 
			case ',':	namecounter=0;
					if(i>0 && (parsing[i-1]==')'|| parsing[i-1]==parsing[i] || isalpha(parsing[i-1]))||numcolon)
						return i+1;
					if(isalpha(parsing[i+1]))return i+2;
					if(parsing[i]!=localeconv()->decimal_point[0]) return i+1;
					numcolon=true;
					if(!isdigit(parsing[i+1])) return i+2;
					break;

			case '(':	++counter;
					if (i>0 && (parsing[i-1]==')' || !isoperator(parsing[i-1])) && !namecounter) return i+1;
					namecounter=0;/* end of fn name in ( */
					break;
			case ')':	--counter;
					if(i==0) return i+1;
					if(parsing[i-1]=='(') return i+1;
					if(!isalnum(parsing[i-1]) && parsing[i-1]!=')')return i-1;
					if(counter<0) return i+1;
					
					namecounter=0;
					break;
			case '*':
			case '/':
			case '^':
			case '%':	if(i==0 || parsing[i-1]=='(' || isoperator(parsing[i-1]) || parsing[i+1]=='\0')
						return i+1;
					namecounter=0;
					break;
			case '+':
			case '-':	if((i>0 && isoperator(parsing[i-1])) || parsing[i+1]=='\0')
						return i+1;
					namecounter=0;
					break;
			case ';':	if((i>0 && (isoperator(parsing[i-1]) || parsing[i-1]==';')) || parsing[i+1]=='\0')
						return i+1;
					namecounter=0;
					if(counter>0 && lastfn>=28)
						break;
			default:	if(isdigit(parsing[i])){
						if (i>0) 
							if(parsing[i-1]==')' || isalpha(parsing[i-1])) return i+1;
					}
					else if (isupper(parsing[i])){
						if(!param_isset(p,parsing[i])) return i+1;
						if(i>0 && parsing[i-1]==')') return i+1;
					}
					else if (islower(parsing[i])){
						++namecounter;
						if(i>0 && parsing[i-1]==')') return i+1;
						if(namecounter==1)
						if(islower(parsing[i+1])){
					   		for (j=1;j<FN_IDENTNUM;++j)
								if (strstr(&parsing[i],fn.name[j])==&parsing[i] && parsing[i+strlen(fn.name[j])]=='('){
									if(lastfn<j)lastfn=j;
									break;
								}
							if (j>=FN_IDENTNUM) return i+1;
						}
						else if (!param_isset(p,parsing[i])) return i+1;else namecounter=0;
					}
					else	return i+1;
					if(i>0 && isalpha(parsing[i]) && isdigit(parsing[i-1])) return i+1;
					if(!islower(parsing[i])) namecounter=0;
		}
	}
	if (counter!=0) return i;/*i==strlen(parsing)*/
	return 0;
}

void preparse(char* parsing){/*todo better call this sth like simplify ? */
/* NOTE: This function turns removed characters to spaces; for deleting those use removespaces() */
	uint32_t i,j;
	int32_t klammerzaehler=0,klammerzaehlerevernotnull=0;
	if(parsing==0 ||strlen(parsing)==0) return;
	
	if(parsing[0]=='(') ++klammerzaehler;
	
	for (i=1;i<strlen(parsing);++i){
		if (parsing[i]<32) parsing[i]=' ';
/* This function part removes globalsourrounding (); before it must be checked if they belong together, unlike (3*x)+(6-1) ->klammerzaehler(evernotnull) */
		if (i+1==strlen(parsing) && klammerzaehlerevernotnull==0 && parsing[0]=='(' && parsing[i]==')')
			parsing[0]=parsing[i]=' ';
		/*subroutine : simplifies +() and -() constructs */
		if (i+1==strlen(parsing) && parsing[1]=='('  && klammerzaehlerevernotnull==0 && parsing[i]==')'){
			if (parsing[0]=='+' || parsing[0]=='-')parsing[1]=parsing[i]=' ';
			/*else */if(parsing[0]=='-'){
/*				if(strdiv(parsing,NULL,"+-","()")==1)
					if(strdiv(parsing,NULL,"/*%","()")==1)
						if(strdiv(parsing,NULL,
				int klammerzaehler=0;
*/
				for (j=2;j<i;++j){
					if (parsing[j]=='(') ++klammerzaehler;
					else if (parsing[j]==')') --klammerzaehler;
					if (klammerzaehler==0){	
						if(parsing[j]=='-') {parsing[j]='+'; continue;}
						else if (parsing[j]=='+') parsing[j]='-';
						else if ((isalnum(parsing[j])|| parsing[j]=='.' || parsing[j]==',') && parsing[j-1]==' ')
		/*invert numbers/vars w/o explicit + sign*/	parsing[j-1]='-';
					}
				}
				parsing[0]=' ';
			}
		}
		if(parsing[i]=='(') ++klammerzaehler;
		else if (parsing[i]==')') --klammerzaehler;
		if (klammerzaehler==0)klammerzaehlerevernotnull=1;
/* This function summarizes neighboring - */
		if (parsing[i-1]=='-' && parsing[i]=='-'){
			parsing[i]='+';
			parsing[i-1]=' ';
		}
/* removal of unnecessary + (at the beginning of the string; if the previous char is no ), digit or parameter */
		if (parsing[i-1]=='+' && (i==1 || (parsing[i-2]!=')' && !isdigit(parsing[i-2]) && !islower(parsing[i-2]) ))) parsing[i-1]=' ';
	}
}

int removespaces(char *parsing){
	char *alt,*neu;
	int count=0;
	if(!parsing)return -1;
	alt=parsing;
	neu=parsing;
	do{
		if(!isspace(*alt))
			*neu++=*alt;
		else ++count;
	}while(*alt++);
	return count;
}

bool canbesimplified(const char* parsing){
	char *endptr;
	if (strlen(parsing)==1 && isalpha(*parsing)) return false;
	if (strlen(parsing)==2 && (*parsing=='-' || *parsing==' ') && isalpha(parsing[1])) return false; 
	strtod(parsing,&endptr);
	if (!endptr || !strlen(endptr)) return false;
	return true;
}

void* recalloc(void* old, size_t old_nmemb, size_t new_nmemb, size_t memb_size){
/* Combines realloc() and calloc()
 * only new mem is nulled */
	void *out;
	if(old_nmemb==new_nmemb) return old;
	out=realloc(old,new_nmemb*memb_size);
	if(old_nmemb<new_nmemb)
		memset((void*)((int)out+(old_nmemb*memb_size)),0,(new_nmemb-old_nmemb)*memb_size);
	return out;

}
